-- $Id: bree-mayor.lua 47 2012-08-28 01:10:46Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

local q_house = game.player:hasQuest('bree-house')
local q_troll = game.player:hasQuest('bree-trolls')
local q_wight = game.player:hasQuest('bree-wights')
local q_nazgul = game.player:hasQuest('bree-nazgul')
local done_stage2 = (q_troll and q_troll:isEnded()) or (q_wight and q_wight:isEnded())

if not q_house then
  newChat {
    id = 'welcome',
    text = [[#LIGHT_GREEN#*Mayor Will Whitfoot looks up as you enter the office and seems delighted to see you.*#WHITE#
Oh, thank goodness!  Someone has finally responded to our signs.]],
    answers = {
      { 'Uhh... signs?', jump = 'sign' },
    },
  }
  newChat {
    id = 'sign',
    text = [[Yes, the signs we put up all over town.  Surely you must have seen them.  There's a gang of thieves terrorizing our town, and we seek the aid of a brave adventurer such as yourself to drive them out.]],
    answers = {
      { "That's awful.  I'll try to help.", jump = 'accept' },
      { 'Yyyeah, sorry, find some other sucker for that job. [leave]' },
    },
  }
  newChat {
    id = 'accept',
    text = [[Excellent!  We believe the blackguards have made their lair in this abandoned building near the center of town.  #LIGHT_GREEN#*He points to a spot on a map of the town hanging on the wall.*#WHITE#  And I doubt they'll leave quietly; you'll probably have to kill them all.  Not that anyone will mourn them, of course.  Best of luck to you!]],
    answers = {
      {
	'Thanks. [leave]',
	action = function(npc, player)
	  player:grantQuest('bree-house')
	  local spot = game.level:pickSpot{ type='quest', subtype='house' }
	  game:onTickEnd(function()
	    game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.ENTER_BREE_HOUSE_QUEST)
	  end)
	end
      }
    },
  }
elseif not q_house:isCompleted() and not q_house:isEnded() then
  newChat {
    id = 'welcome',
    text = [[Ah, hello again.  How goes your attempt on the thieves?]],
    answers = {
      { "I'm, uh, still scouting out their hideout. [leave]" }
    },
  }
elseif q_house:isCompleted() then
  newChat {
    id = 'welcome',
    text = [[Splendid work, brave @playername@!  I'm already being told that the streets of Bree are safe again.  As a show of our official gratitude, I'm signing over to you the deed for the building they were using as a hideout.  You can use it as a house if you like.]],
    answers = {
      {
	'Thank you.',
	action = function(npc, player)
	  player:setQuestStatus('bree-house', engine.Quest.DONE)
	  local spot = game.level:pickSpot{ type='quest', subtype='house' }
	  game:onTickEnd(function()
	    game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.PERMA_WALL)
	    local e = game.zone:makeEntityByName(game.level, 'trap', 'BREE_HOME')
	    game.zone:addEntity(game.level, e, 'trap', spot.x, spot.y, engine.Map.TRAP)
	  end)
	end
      }
    }
  }
elseif q_house:isEnded() and q_house.next_quest then
  newChat {
    id = 'welcome',
    text = [[Ah, hello again, esteemed @playername@!  How's the new house?  Actually, since you're here, we have another problem that might be right up your alley.]],
    answers = {
      { 'What sort of problem?', jump = q_house.next_quest },
    },
  }
  newChat {
    id = 'jump_troll',
    text = [[Well, apparently some forest trolls have set up camp in the forest southeast of town, and they're sending raiding parties to kill our townsfolk.  Far beyond the means of our own Shirriffs to drive them off, but I'm sure a mighty adventurer such as yourself can make quick work of them.]],
    answers = {
      { "I'm in.", jump = 'troll_accept' },
      { 'Yeah, no. [leave]' },
    },
  }
  newChat {
    id = 'jump_wight',
    text = [[Well, apparently something particularly nasty has set up camp near the Barrow Downs.  Much worse than the riffraff we usually see from the Downs, though those are certainly bad enough; no, these are said to be the walking dead. #{italic}#*shudder*#{normal}#  Some of our townsfolk have disappeared in that general area.  I'm sure a mighty adventurer such as yourself can make quick work of it, though.]],
    answers = {
      {
	"I'm in.",
	action = function(npc, player)
	  player:grantQuest('bree-wights')
	  local spot = game.level:pickSpot{ type='quest', subtype='wights' }
	  q_house.next_quest = nil
	  game:onTickEnd(function()
	    game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.ENTER_BREE_WIGHTS_QUEST)
	  end)
	end
      },
      { 'Yeah, no. [leave]' },
    },
  }
  newChat {
    id = 'troll_accept',
    text = [[Excellent!  They seem to keep hidden during the day, so you should probably look for them at night.  And the few survivors of our scouting parties report that these trolls have one or two mighty Elven weapons in their possession.  Bring one of those back to show you've gotten rid of them.]],
    answers = {
      {
	'Done and done.',
	action = function(npc, player)
	  player:grantQuest('bree-trolls')
	  local spot = game.level:pickSpot{ type='quest', subtype='trolls' }
	  q_house.next_quest = nil
	  -- TODO Make this only appear at night.
	  game:onTickEnd(function()
	    game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.ENTER_BREE_TROLLS_QUEST)
	  end)
	end
      }
    },
  }
elseif q_troll and not q_troll:isCompleted() and not q_troll:isEnded() then
  newChat {
    id = 'welcome',
    text = [[Ah, hello again.  Have you ferreted out those nasty trolls yet?]],
    answers = {
      { "Um, no, not yet. [leave]" }
    },
  }
elseif q_wight and not q_wight:isCompleted() and not q_wight:isEnded() then
  newChat {
    id = 'welcome',
    text = [[Ah, hello again.  Have you ferreted out those nasty undead things yet?]],
    answers = {
      { "Um, no, not yet. [leave]" }
    },
  }
elseif q_troll and q_troll:isCompleted() then
  newChat {
    id = 'welcome',
    text = [[Ah, splendid!  I had heard there'd been some brouhaha in that part of the forest, and here you are, safe and sound.  Ooh, yes, that #{italic}#is#{normal}# a nice sword.  I trust the trolls are dead, then.  Good, good!  No, no, you keep the sword.  Lord knows what I'd do with it, probably just hang it on a wall or something, but it should serve you well in your adventures.]],
    answers = {
      {
	'Thank you. [leave]',
	action = function(npc, player)
	  player:setQuestStatus('bree-trolls', engine.Quest.DONE)
	end
      },
    },
  }
elseif q_wight and q_wight:isCompleted() then
  newChat {
    id = 'welcome',
    text = [[Ah, splendid!  Already the townsfolk are telling me that the air in the area is much less oppressively grim.  Once again, thank you, noble adventurer.  Eugh!  I suppose that foul rag is what the vile thing was wearing.  Please do take it wih you when you leave, don't want it fouling up the place.  You should have it checked out, though; if the wights had it, perhaps it is magical and will serve you well in your adventures, if you can bear to wear it.]],
    answers = {
      {
	'Thank you. [leave]',
	action = function(npc, player)
	  player:setQuestStatus('bree-wights', engine.Quest.DONE)
	end
      },
    },
  }
elseif done_stage2 and not q_nazgul and game.player.level < 30 then
  newChat {
    id = 'welcome',
    text = [[Ah, hello again, brave @playername@.  Now, this is awkward...  You see, there is a matter I'd like to have looked into, but it's a particularly nasty one and, well, from what I've heard of your exploits, I'm afraid it may be a bit beyond you at the moment.  But no worries; I hear you're getting stronger every day.  Go out into the wild and gain some more experience, and perhaps later you can help me.]],
    answers = {
      { 'Um... okay. [leave]' },
    },
  }
elseif done_stage2 and not q_nazgul and game.player.level >= 30 then
  newChat {
    id = 'welcome',
    text = [[Ah, my good friend @playername@!  I was just thinking of you.  You see, there's been a #{italic}#very#{normal}# shady looking fellow spotted several times skulking about town asking questions.  Long black cloak, nasty oppressive aura, scares the willikers out of the people who run across him.  And worse, people seem to be falling ill in the areas where he's been seen.  I was hoping I could persuade you to see to him.  I give you fair warning, though, he's reported to be rather powerful; you'll want to approach him with caution.]],
    answers = {
      {
	"I'll see what I can do about him.",
	action = function(npc, player)
	  player:grantQuest('bree-nazgul')
	  -- TODO Place Uvatha on the map.
	end
      },
      { 'Do I #{italic}#look#{normal}# crazy?  No chance.' },
    },
  }
elseif q_nazgul and not q_nazgul:isCompleted() and not q_nazgul:isEnded() then
  newChat {
    id = 'welcome',
    text = [[Eep!  #LIGHT_GREEN#*The mayor ducks behind his desk, then peeks up over it*#WHITE# Oh, it's just you. #{italic}#*whew*#{normal}#  Any luck with that shady fellow?]],
    answers = {
      { "I haven't encountered him yet." },
    },
  }
elseif q_nazgul and q_nazgul:isCompleted() then
  newChat {
    id = 'welcome',
    text = [[Well done, brave @playername@!  I saw a bit of the fight myself, and it looked quite harrowing, I must say!  Speaking of which, I have a little something for you, something a Ranger once gave me.  Now, these may look like ordinary leaves, but they're called kingsfoil, and they're reputed to have some rather impressive medicinal effects if applied properly.  Indeed, some of those who had fallen ill to that scoundrel have used it to quite good effect.  I imagine you could use some yourself right about now after a fight like that.]],
    answers = {
      {
	'Thank you.',
	action = function(npc, player)
	  player:setQuestStatus('bree-nazgul', engine.Quest.DONE)
	  -- TODO Give the player some Athelas.
	end
      }
    },
  }
else
  newChat {
    id = 'welcome',
    text = [[And hello again, brave @playername@.  No doubt come looking for more good works to perform for the town?  Now, that's the mark of a truly noble adventurer, I always say.  Well, sad to say, nothing seems to be troubling our fair town at the moment.  Sad for you, that is; it's certainly a relief for us to have a break from all those harrowing misfortunes you've so ably helped us through, much to our eternal gratitude, of course.]],
    answers = {
      { 'Okay. [leave]' },
    },
  }
end

return 'welcome'
