-- $Id: damage_types.lua 43 2012-08-27 04:46:39Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Damage types extracted from T2's melee1.c

-- The basic stuff used to damage a grid
setDefaultProjector(function(src, x, y, type, dam, p)
  p = p or {}
  local atk_msg = p.atk_msg or ' hits %s'
  local fmt = src.name:capitalize()..atk_msg..' for %d %s%s#LAST# damage.'
  local fmt_0dam = src.name:capitalize()..atk_msg..'.'
  local dt = DamageType:get(type)
  local dt_color = dt.text_color or '#AAAAAA#'

  local target = game.level.map(x, y, Map.ACTOR)
  if target then
    local flash = game.flash.NEUTRAL
    if target == game.player then flash = game.flash.BAD end
    if src == game.player then flash = game.flash.GOOD end

    if dam == 0 and p.silent_0dam then
      game.logSeen(target, flash, fmt_0dam, target.name)
    else
      game.logSeen(target, flash, fmt, target.name, dam, dt_color, dt.name)
    end
    local sx, sy = game.level.map:getTileToScreen(x, y)
    if target:takeHit(dam, src) then
      if src == game.player or target == game.player then
	game.flyers:add(sx, sy, 30, (rng.range(0,2)-1) * 0.5, -3, "Kill!", {255,0,255})
      end
    else
      local flyer_color = src == game.player and {0,255,0} or target == game.player and {255,0,0} or nil
      if flyer_color then
	game.flyers:add(sx, sy, 30, (rng.range(0,2)-1) * 0.5, -3, tostring(-math.ceil(dam)), flyer_color)
      end
    end
    return dam
  end
  return 0
end)

newDamageType {
  name = 'physical',
  type = 'PHYSICAL',
  power = 60,
}

newDamageType {
  name = 'poison',
  type = 'POISON',
  power = 5,
}

newDamageType {
  name = 'un_bonus',
  type = 'UN_BONUS',
  power = 20,
}

newDamageType {
  name = 'un_power',
  type = 'UN_POWER',
  power = 15,
}

newDamageType {
  name = 'eat_gold',
  type = 'EAT_GOLD',
  power = 5,
  projector = function(src, x, y, type, dam, p)
    local realdam = DamageType.defaultProjector(src, x, y, type, dam, p)
    -- Slight Hack(TM):  Apparently in T2 gold can only be stolen from the
    -- player.
    -- TODO Rebelling symbiotes shouldn't be able to steal gold either.
    local target = game.level.map(x, y, Map.ACTOR)
    local blink_chance = 0
    if target and target == game.player then
      local save = target.level + target:getStatRelated('dex_save')
      -- TODO Disallow if paralyzed.
      if rng.percent(save) then
	game.log('You quickly protect your money pouch!')
	blink_chance = 3
      else
	local amt = math.floor((target.gold or 0)/10) + rng.range(1, 25)
	amt = math.max(amt, 2)
	if amt > 5000 then
	  amt = math.floor(target.gold/20) + rng.range(1, 3000)
	end
	amt = math.min(amt, target.gold)
	if amt <= 0 then
	  game.log('Nothing was stolen.')
	else
	  local gold = game.zone:makeEntityByName(game.level, 'object', 'GOLD')
	  gold.money_value = amt
	  src:addObject(src.INVEN_INVEN, gold)
	  local inven = src:getInven('INVEN') or {}
	  game.log('Your purse feels lighter.')
	  local pfx = amt < target.gold and ('%d coins'):format(amt) or 'All your coins'
	  game.log(pfx..' were stolen!')
	  target:adjGold(-amt)
	end
	blink_chance = 1
      end
    else
      blink_chance = 2
    end
    if blink_chance > 0 and rng.range(1, blink_chance) == 1 then
      game.logSeen(src, '%s flees laughing!', src.name)
      src:teleportRandom(src.x, src.y, 45, 20)
    end
  end
}

newDamageType {
  name = 'eat_item',
  type = 'EAT_ITEM',
  power = 5,
}

newDamageType {
  name = 'eat_food',
  type = 'EAT_FOOD',
  power = 5,
}

newDamageType {
  name = 'eat_lite',
  type = 'EAT_LITE',
  power = 5,
}

newDamageType {
  name = 'acid',
  type = 'ACID',
  power = 0,
}

newDamageType {
  name = 'elec',
  type = 'ELEC',
  power = 10,
}

newDamageType {
  name = 'fire',
  type = 'FIRE',
  power = 10,
}

newDamageType {
  name = 'cold',
  type = 'COLD',
  power = 10,
}

newDamageType {
  name = 'blind',
  type = 'BLIND',
  power = 2,
}

newDamageType {
  name = 'confuse',
  type = 'CONFUSE',
  power = 10,
}

newDamageType {
  name = 'terrify',
  type = 'TERRIFY',
  power = 10,
}

newDamageType {
  name = 'paralyze',
  type = 'PARALYZE',
  power = 2,
}

newDamageType {
  name = 'lose_str',
  type = 'LOSE_STR',
  power = 0,
}

newDamageType {
  name = 'lose_int',
  type = 'LOSE_INT',
  power = 0,
}

newDamageType {
  name = 'lose_wis',
  type = 'LOSE_WIS',
  power = 0,
}

newDamageType {
  name = 'lose_dex',
  type = 'LOSE_DEX',
  power = 0,
}

newDamageType {
  name = 'lose_con',
  type = 'LOSE_CON',
  power = 0,
}

newDamageType {
  name = 'lose_chr',
  type = 'LOSE_CHR',
  power = 0,
}

newDamageType {
  name = 'lose_all',
  type = 'LOSE_ALL',
  power = 2,
}

newDamageType {
  name = 'shatter',
  type = 'SHATTER',
  power = 60,
}

newDamageType {
  name = 'exp_10',
  type = 'EXP_10',
  power = 5,
}

newDamageType {
  name = 'exp_20',
  type = 'EXP_20',
  power = 5,
}

newDamageType {
  name = 'exp_40',
  type = 'EXP_40',
  power = 5,
}

newDamageType {
  name = 'exp_80',
  type = 'EXP_80',
  power = 5,
}

newDamageType {
  name = 'disease',
  type = 'DISEASE',
  power = 5,
}

newDamageType {
  name = 'time',
  type = 'TIME',
  power = 5,
}

newDamageType {
  name = 'sanity',
  type = 'SANITY',
  power = 60,
}

newDamageType {
  name = 'hallu',
  type = 'HALLU',
  power = 10,
}

newDamageType {
  name = 'parasite',
  type = 'PARASITE',
  power = 5,
}

newDamageType {
  name = 'abomination',
  type = 'ABOMINATION',
  power = 20,
}
