-- $Id: all.lua 41 2012-08-27 04:44:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Grid entities generated from T2's f_info.txt.

local function try_open_locked_door(self, e)
  -- TODO handle NPCs with OPEN_DOOR flag
  if not e.skill_disarm then return false, false end
  local chance = e.skill_disarm - self.door_lock_power * 4
  -- TODO adjust for blindness/confusion/hallucination
  chance = math.max(chance, 2)
  if rng.percent(chance) then
    game.log('You have picked the lock.')
    e:gainExp(1)
    return true, false
  else
    game.log('You failed to pick the lock.')
    return false, true
  end
end

local function try_open_jammed_door(self, e)
  game.log('The door appears to be stuck.')
  return false, false
end

newEntity {
  define_as = 'NOTHING',
  name = 'nothing',
  display = ' ',
  color = colors.C_w,
}

newEntity {
  define_as = 'FLOOR',
  name = 'open floor',
  display = '.',
  color = colors.C_w,
}

newEntity {
  define_as = 'FOUNTAIN',
  name = 'fountain',
  display = '_',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'GLYPH_OF_WARDING',
  name = 'glyph of warding',
  display = ';',
  color = colors.C_y,
  always_remember = true,
}

newEntity {
  define_as = 'OPEN_DOOR',
  name = 'open door',
  display = "'",
  color = colors.C_U,
  always_remember = true,
  door_closed = 'DOOR',
}

newEntity {
  define_as = 'BROKEN_DOOR',
  name = 'broken door',
  display = "'",
  color = colors.C_U,
  always_remember = true,
}

newEntity {
  define_as = 'UP',
  name = 'up staircase',
  display = '<',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'DOWN',
  name = 'down staircase',
  display = '>',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'QUEST_ENTRANCE',
  name = 'quest entrance',
  display = '>',
  color = colors.C_y,
  always_remember = true,
}

newEntity {
  define_as = 'QUEST_EXIT',
  name = 'quest exit',
  display = '<',
  color = colors.C_y,
  always_remember = true,
}

newEntity {
  define_as = 'QUEST_DOWN_LEVEL',
  name = 'quest down level',
  display = '>',
  color = colors.C_r,
  always_remember = true,
}

newEntity {
  define_as = 'QUEST_UP_LEVEL',
  name = 'quest up level',
  display = '<',
  color = colors.C_r,
  always_remember = true,
}

newEntity {
  define_as = 'TOWN_EXIT',
  name = 'town exit',
  display = '>',
  color = colors.C_g,
  always_remember = true,
}

newEntity {
  define_as = 'SHAFT_DOWN',
  name = 'shaft down',
  display = '>',
  color = colors.C_U,
  always_remember = true,
}

newEntity {
  define_as = 'SHAFT_UP',
  name = 'shaft up',
  display = '<',
  color = colors.C_U,
  always_remember = true,
}

newEntity {
  define_as = 'EMPTY_FOUNTAIN',
  name = 'fountain',
  display = '_',
  color = colors.C_D,
  always_remember = true,
}

newEntity {
  define_as = 'WEB',
  name = 'web',
  display = '+',
  color = colors.C_y,
  always_remember = true,
}

newEntity {
  define_as = 'TRAP',
  name = 'trap',
  display = '^',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'DOOR',
  name = 'door',
  display = '+',
  color = colors.C_U,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
  door_opened = 'OPEN_DOOR',
  is_openable = true,
}

newEntity {
  base = 'DOOR',
  define_as = 'LOCKED_DOOR_BASE',
  name = 'locked door',
  try_open_door = try_open_locked_door,
}

newEntity {
  base = 'DOOR',
  define_as = 'JAMMED_DOOR_BASE',
  name = 'jammed door',
  try_open_door = try_open_jammed_door,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_1',
  door_lock_power = 1,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_2',
  door_lock_power = 2,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_3',
  door_lock_power = 3,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_4',
  door_lock_power = 4,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_5',
  door_lock_power = 5,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_6',
  door_lock_power = 6,
}

newEntity {
  base = 'LOCKED_DOOR_BASE',
  define_as = 'LOCKED_DOOR_POWER_7',
  door_lock_power = 7,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_0',
  door_jam_power = 0,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_1',
  door_jam_power = 1,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_2',
  door_jam_power = 2,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_3',
  door_jam_power = 3,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_4',
  door_jam_power = 4,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_5',
  door_jam_power = 5,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_6',
  door_jam_power = 6,
}

newEntity {
  base = 'JAMMED_DOOR_BASE',
  define_as = 'JAMMED_DOOR_POWER_7',
  door_jam_power = 7,
}

newEntity {
  define_as = 'SECRET_DOOR',
  name = 'secret door',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PILE_OF_RUBBLE',
  name = 'pile of rubble',
  display = ':',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'MAGMA_VEIN',
  name = 'magma vein',
  display = '%',
  color = colors.C_s,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'QUARTZ_VEIN',
  name = 'quartz vein',
  display = '%',
  color = colors.C_w,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'MAGMA_VEIN',
  name = 'magma vein',
  display = '%',
  color = colors.C_s,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'QUARTZ_VEIN',
  name = 'quartz vein',
  display = '%',
  color = colors.C_w,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'MAGMA_VEIN_WITH_TREASURE',
  name = 'magma vein with treasure',
  display = '*',
  color = colors.C_o,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'QUARTZ_VEIN_WITH_TREASURE',
  name = 'quartz vein with treasure',
  display = '*',
  color = colors.C_o,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'WALL',
  name = 'granite wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'WALL',
  name = 'granite wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'WALL',
  name = 'granite wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'WALL',
  name = 'granite wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'EXPLOSIVE_RUNE',
  name = 'explosive rune',
  display = '*',
  color = colors.C_R,
}

newEntity {
  define_as = 'STRAIGHT_ROAD_STARTPOINT',
  name = 'Straight Road startpoint',
  display = '*',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'SECTION_OF_THE_STRAIGHT_ROAD',
  name = 'section of the Straight Road',
  display = '*',
  color = colors.C_B,
  always_remember = true,
}

newEntity {
  define_as = 'SECTION_OF_THE_STRAIGHT_ROAD',
  name = 'section of the Straight Road',
  display = '*',
  color = colors.C_b,
  always_remember = true,
}

newEntity {
  define_as = 'SECTION_OF_THE_STRAIGHT_ROAD',
  name = 'section of the Straight Road',
  display = '*',
  color = colors.C_B,
  always_remember = true,
}

newEntity {
  define_as = 'SECTION_OF_THE_STRAIGHT_ROAD',
  name = 'section of the Straight Road',
  display = '*',
  color = colors.C_b,
  always_remember = true,
}

newEntity {
  define_as = 'SECTION_OF_THE_STRAIGHT_ROAD',
  name = 'section of the Straight Road',
  display = '*',
  color = colors.C_W,
  always_remember = true,
}

newEntity {
  define_as = 'SECTION_OF_THE_STRAIGHT_ROAD__DISCHARGED_',
  name = 'section of the Straight Road (discharged)',
  display = '*',
  color = colors.C_W,
  always_remember = true,
}

newEntity {
  define_as = 'STRAIGHT_ROAD_EXIT',
  name = 'Straight Road exit',
  display = '*',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'CORRUPTED_SECTION_OF_THE_STRAIGHT_ROAD',
  name = 'corrupted section of the Straight Road',
  display = '*',
  color = colors.C_D,
  always_remember = true,
}

newEntity {
  define_as = 'BUILDING',
  name = 'Building',
  display = '1',
  color = colors.C_U,
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'PERMA_WALL',
  name = 'permanent wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'STREAM_OF_SHALLOW_WATER',
  name = 'stream of shallow water',
  display = '~',
  color = colors.C_B,
  always_remember = true,
}

newEntity {
  define_as = 'POOL_OF_DEEP_LAVA',
  name = 'pool of deep lava',
  display = '.',
  color = colors.C_R,
  always_remember = true,
}

newEntity {
  define_as = 'STREAM_OF_SHALLOW_LAVA',
  name = 'stream of shallow lava',
  display = '.',
  color = colors.C_r,
  always_remember = true,
}

newEntity {
  define_as = 'DARK_PIT',
  name = 'dark pit',
  display = '#',
  color = colors.C_D,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'DIRT',
  name = 'dirt',
  display = '.',
  color = colors.C_U,
}

newEntity {
  define_as = 'GRASS',
  name = 'patch of grass',
  display = '.',
  color = colors.C_G,
}

newEntity {
  define_as = 'ICE',
  name = 'ice',
  display = '.',
  color = colors.C_W,
}

newEntity {
  define_as = 'SAND',
  name = 'sand',
  display = '.',
  color = colors.C_y,
}

newEntity {
  define_as = 'DEAD_TREE',
  name = 'dead tree',
  display = '#',
  color = colors.C_D,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'ASH',
  name = 'ash',
  display = '.',
  color = colors.C_s,
}

newEntity {
  define_as = 'MUD',
  name = 'mud',
  display = '.',
  color = colors.C_u,
}

newEntity {
  define_as = 'ICE_WALL',
  name = 'ice wall',
  display = '#',
  color = colors.C_W,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'TREE',
  name = 'tree',
  display = '#',
  color = colors.C_G,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'MOUNTAIN_CHAIN',
  name = 'mountain chain',
  display = '^',
  color = colors.C_U,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'SANDWALL',
  name = 'sandwall',
  display = '#',
  color = colors.C_y,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'SANDWALL',
  name = 'sandwall',
  display = '%',
  color = colors.C_y,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'SANDWALL_WITH_TREASURE',
  name = 'sandwall with treasure',
  display = '*',
  color = colors.C_o,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'HIGH_MOUNTAIN_CHAIN',
  name = 'high mountain chain',
  display = '^',
  color = colors.C_W,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'NETHER_MIST',
  name = 'nether mist',
  display = '.',
  color = colors.C_v,
}

newEntity {
  define_as = 'MOLTEN_GLASS_WALL',
  name = 'molten glass wall',
  display = '.',
  color = colors.C_B,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'VOID_JUMPGATE',
  name = 'Void Jumpgate',
  display = '+',
  color = colors.C_v,
  always_remember = true,
}

newEntity {
  define_as = 'ALTAR_OF_BEING',
  name = 'Altar of Being',
  display = '0',
  color = colors.C_W,
  always_remember = true,
}

newEntity {
  define_as = 'ALTAR_OF_WINDS',
  name = 'Altar of Winds',
  display = '0',
  color = colors.C_B,
  always_remember = true,
}

newEntity {
  define_as = 'ALTAR_OF_FORCE',
  name = 'Altar of Force',
  display = '0',
  color = colors.C_R,
  always_remember = true,
}

newEntity {
  define_as = 'ALTAR_OF_DARKNESS',
  name = 'Altar of Darkness',
  display = '0',
  color = colors.C_D,
  always_remember = true,
}

newEntity {
  define_as = 'ALTAR_OF_NATURE',
  name = 'Altar of Nature',
  display = '0',
  color = colors.C_g,
  always_remember = true,
}

newEntity {
  define_as = 'FLOOR',
  name = 'open floor',
  display = '.',
  color = colors.C_w,
}

newEntity {
  define_as = 'UNDERGROUND_TUNNEL',
  name = 'Underground Tunnel',
  display = '#',
  color = colors.C_s,
  always_remember = true,
}

newEntity {
  define_as = 'STREAM_OF_TAINTED_WATER',
  name = 'stream of tainted water',
  display = '~',
  color = colors.C_u,
  always_remember = true,
}

newEntity {
  define_as = 'MONSTER_TRAP',
  name = 'monster trap',
  display = ';',
  color = colors.C_v,
}

newEntity {
  define_as = 'VOID_JUMPGATE',
  name = 'Void Jumpgate',
  display = '+',
  color = colors.C_v,
  always_remember = true,
}

newEntity {
  define_as = 'LAVA_WALL',
  name = 'lava wall',
  display = '#',
  color = colors.C_R,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'GREAT_FIRE',
  name = 'Great Fire',
  display = '%',
  color = colors.C_v,
  always_remember = true,
}

newEntity {
  define_as = 'PATH_TO_THE_NEXT_AREA',
  name = 'path to the next area',
  display = '>',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'PATH_TO_THE_PREVIOUS_AREA',
  name = 'path to the previous area',
  display = '<',
  color = colors.C_w,
  always_remember = true,
}

newEntity {
  define_as = 'FIELD',
  name = 'field',
  display = ':',
  color = colors.C_g,
  always_remember = true,
}

newEntity {
  define_as = 'EKKAIA__THE_ENCIRCLING_SEA',
  name = 'Ekkaia, the Encircling Sea',
  display = '*',
  color = colors.C_b,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'VOID',
  name = 'void',
  display = ' ',
  color = colors.C_d,
}

newEntity {
  define_as = 'DEEP_WATER',
  name = 'pool of deep water',
  display = '~',
  color = colors.C_b,
  always_remember = true,
}

newEntity {
  define_as = 'GLASS_WALL',
  name = 'glass wall',
  display = '.',
  color = colors.C_B,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'ILLUSION_WALL',
  name = 'illusion wall',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'GRASS_ROOF',
  name = 'Grass roof',
  display = '#',
  color = colors.C_y,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'GRASS_ROOF_TOP',
  name = 'grass roof top',
  display = '#',
  color = colors.C_y,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'GRASS_ROOF_CHIMNEY',
  name = 'grass roof chimney',
  display = '#',
  color = colors.C_y,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'BRICK_ROOF',
  name = 'brick roof',
  display = '#',
  color = colors.C_r,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'BRICK_ROOF_TOP',
  name = 'brick roof top',
  display = '#',
  color = colors.C_r,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'BRICK_ROOF_CHIMNEY',
  name = 'brick roof chimney',
  display = '#',
  color = colors.C_r,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'WINDOW',
  name = 'window',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'SMALL_WINDOW',
  name = 'small window',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'RAIN_BARREL',
  name = 'rain barrel',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'GRASS_WITH_FLOWERS',
  name = 'grass with flowers',
  display = ';',
  color = colors.C_G,
}

newEntity {
  define_as = 'COBBLESTONE_ROAD',
  name = 'cobblestone road',
  display = '.',
  color = colors.C_w,
}

newEntity {
  define_as = 'COBBLESTONE_WITH_OUTLET',
  name = 'cobblestone with outlet',
  display = '.',
  color = colors.C_w,
}

newEntity {
  define_as = 'SMALL_TREE',
  name = 'small tree',
  display = '#',
  color = colors.C_g,
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'TOWN',
  name = 'town',
  display = '*',
  color = colors.C_w,
}

newEntity {
  define_as = 'UNDERGROUND_TUNNEL',
  name = 'Underground Tunnel',
  display = '^',
  color = colors.C_U,
  always_remember = true,
}

newEntity {
  define_as = 'A_BLAZING_FIRE',
  name = 'a blazing fire',
  display = '%',
  color = colors.C_y,
  always_remember = true,
}

newEntity {
  define_as = 'PILE_OF_RUBBLE',
  name = 'pile of rubble',
  display = ':',
  color = colors.C_w,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'ROCKY_GROUND',
  name = 'rocky ground',
  display = '.',
  color = colors.C_s,
}

newEntity {
  define_as = 'CLOUD_LIKE_VAPOUR',
  name = 'cloud-like vapour',
  display = '.',
  color = colors.C_W,
}

newEntity {
  define_as = 'CONDENSING_WATER',
  name = 'condensing water',
  display = '~',
  color = colors.C_B,
  always_remember = true,
}

newEntity {
  define_as = 'DENSE_MIST',
  name = 'dense mist',
  display = '#',
  color = colors.C_w,
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'HAIL_STONE_WALL',
  name = 'hail-stone wall',
  display = '#',
  color = colors.C_W,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'DEAD_SMALL_TREE',
  name = 'dead small tree',
  display = '#',
  color = colors.C_D,
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'COPPER_PILLAR',
  name = 'copper pillar',
  display = '#',
  color = colors.C_u,
  does_block_move = true,
  can_pass = {pass_wall=1},
  block_sight = true,
  always_remember = true,
}

newEntity {
  define_as = 'ETHEREAL_WALL',
  name = 'ethereal wall',
  display = '.',
  color = colors.C_w,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'GLACIAL_WALL',
  name = 'glacial wall',
  display = '#',
  color = colors.C_B,
  block_sight = true,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}

newEntity {
  define_as = 'BATTLEMENT',
  name = 'battlement',
  display = '#',
  color = colors.C_w,
  does_block_move = true,
  can_pass = {pass_wall=1},
  always_remember = true,
}
