local AttackType = require 'mod.class.AttackType'

local bodies = {
  humanoid = { MAIN_HAND=1, OFF_HAND=1, BODY=1, CLOAK=1, LAUNCHER=1, QUIVER=1, GLOVES=1, BOOTS=1, HELM=1, RING=2, AMULET=1, LITE=1, TOOL=1, SYMBIOTE=1, INVEN=23 },
  quadruped = { MAIN_HAND=0, OFF_HAND=0, BODY=1, CLOAK=1, LAUNCHER=0, QUIVER=1, GLOVES=0, BOOTS=0, HELM=1, RING=2, AMULET=1, LITE=1, TOOL=0, SYMBIOTE=1, INVEN=23 },
}

newEntity {
  define_as = 'SCRAWNY_CAT',
  name = [[Scrawny cat]],
  display = 'f',
  color = colors.C_U,
  desc = [[A skinny little furball with sharp claws and a menacing look.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=25,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 2,
  ac = 1,
  max_life = resolvers.dice(1, 2),
  rarity = 3,
  level = 0,
  exp_worth = 0,
  flags = {
    ANIMAL = true,
    WILD_TOWN = true,
    MORTAL = true,
    WILD_ONLY = true,
    DROP_SKELETON = true,
    WILD_TOO = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={1,1} },
  },
  body = bodies.quadruped,
  speed = 0,
  vision = 30,
  alertness = 10,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'AGENT_OF_THE_BLACK_MARKET',
  name = [[Agent of the black market]],
  display = 't',
  color = colors.C_b,
  desc = [[He 'finds' new wares for the Black Market. From unwary adventurers...]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 8,
  ac = 8,
  max_life = resolvers.dice(2, 8),
  rarity = 1,
  level = 0,
  exp_worth = 0,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    WILD_SWAMP = true,
    MALE = true,
    WILD_WOOD = true,
    DROP_60 = true,
    EVIL = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
    WILD_TOWN = true,
    WILD_MOUNTAIN = true,
    DROP_SKELETON = true,
    BASH_DOOR = true,
    HAS_LITE = true,
    OPEN_DOOR = true,
    TAKE_ITEM = true,
  },
  drop = { treasure=25, combat=50, magic=20, tool=5 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.HIT, dam={1,6} },
    { type=DamageType.EAT_ITEM, method=AttackType.TOUCH },
    { method=AttackType.INSULT },
  },
  body = bodies.humanoid,
  speed = 0,
  vision = 10,
  alertness = 99,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'MEAN_LOOKING_MERCENARY',
  name = [[Mean-looking mercenary]],
  display = 't',
  color = colors.C_r,
  desc = [[No job is too low for him.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=50,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 8,
  ac = 20,
  max_life = resolvers.dice(5, 8),
  rarity = 1,
  level = 0,
  exp_worth = 0,
  flags = {
    WILD_TOWN = true,
    MORTAL = true,
    WILD_ONLY = true,
    MALE = true,
    WILD_WOOD = true,
    DROP_SKELETON = true,
    BASH_DOOR = true,
    EVIL = true,
    DROP_90 = true,
    HAS_LITE = true,
    OPEN_DOOR = true,
    TAKE_ITEM = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=100, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.HIT, dam={1,10} },
  },
  body = bodies.humanoid,
  speed = 0,
  vision = 10,
  alertness = 250,
  resolvers.mon_global_speed()
}


-- seen dam types:  EAT_ITEM PHYSICAL
-- seen hit msgs:  CLAW TOUCH INSULT HIT
-- seen body types:
--    quadruped [0,1,0,2,1,0]:  1 (first seen on Scrawny cat)
--    humanoid [1,1,1,2,1,1]:  2 (first seen on Agent of the black market)
