local AttackType = require 'mod.class.AttackType'

local bodies = {
  humanoid = { MAIN_HAND=1, OFF_HAND=1, BODY=1, CLOAK=1, LAUNCHER=1, QUIVER=1, GLOVES=1, BOOTS=1, HELM=1, RING=2, AMULET=1, LITE=1, TOOL=1, SYMBIOTE=1, INVEN=23 },
  quadruped = { MAIN_HAND=0, OFF_HAND=0, BODY=1, CLOAK=1, LAUNCHER=0, QUIVER=1, GLOVES=0, BOOTS=0, HELM=1, RING=2, AMULET=1, LITE=1, TOOL=0, SYMBIOTE=1, INVEN=23 },
  none = { MAIN_HAND=0, OFF_HAND=0, BODY=0, CLOAK=0, LAUNCHER=0, QUIVER=0, GLOVES=0, BOOTS=0, HELM=0, RING=0, AMULET=0, LITE=0, TOOL=0, SYMBIOTE=0, INVEN=23 },
  winged = { MAIN_HAND=0, OFF_HAND=1, BODY=1, CLOAK=1, LAUNCHER=0, QUIVER=1, GLOVES=1, BOOTS=0, HELM=1, RING=0, AMULET=1, LITE=1, TOOL=1, SYMBIOTE=1, INVEN=23 },
}

newEntity {
  define_as = 'CHAFFINCH',
  name = [[Chaffinch]],
  display = 'B',
  color = colors.C_r,
  desc = [[Utterly harmless, except when angry.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=25,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 1,
  ac = 1,
  max_life = resolvers.dice(1, 1),
  rarity = 3,
  level = 0,
  exp_worth = 0,
  flags = {
    ANIMAL = true,
    MORTAL = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    DROP_SKELETON = true,
    IMPRESED = true,
    CAN_FLY = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    HAS_EGG = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,1} },
  },
  body = bodies.winged,
  speed = 0,
  vision = 30,
  alertness = 10,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'WILD_RABBIT',
  name = [[Wild rabbit]],
  display = 'r',
  color = colors.C_U,
  desc = [[It is not a carnivore, but will defend itself if you stray too close.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=50,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 2,
  ac = 1,
  max_life = resolvers.dice(1, 2),
  rarity = 3,
  level = 0,
  exp_worth = 0,
  flags = {
    ANIMAL = true,
    MORTAL = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    DROP_SKELETON = true,
    WILD_GRASS = true,
    DROP_CORPSE = true,
    BASEANGBAND = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,1} },
  },
  body = bodies.quadruped,
  speed = 0,
  vision = 30,
  alertness = 10,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'WOODSMAN',
  name = [[Woodsman]],
  display = 't',
  color = colors.C_g,
  desc = [[He has a strong axe with a sharp edge.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=25,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 3,
  ac = 1,
  max_life = resolvers.dice(3, 3),
  rarity = 1,
  level = 0,
  exp_worth = 0,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    MALE = true,
    DROP_60 = true,
    WILD_WOOD = true,
    ONLY_GOLD = true,
    DROP_SKELETON = true,
    BASH_DOOR = true,
    OPEN_DOOR = true,
    HAS_LITE = true,
    TAKE_ITEM = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.HIT, dam={1,6} },
  },
  body = bodies.humanoid,
  speed = 0,
  vision = 10,
  alertness = 255,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'GIANT_GREEN_FROG',
  name = [[Giant green frog]],
  display = 'R',
  color = colors.C_g,
  desc = [[It is as big as a wolf.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=25,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 8,
  ac = 8,
  max_life = resolvers.dice(2, 8),
  rarity = 1,
  level = 2,
  exp_worth = 6,
  flags = {
    ANIMAL = true,
    WILD_SHORE = true,
    MORTAL = true,
    WILD_SWAMP = true,
    WILD_ONLY = true,
    BASH_DOOR = true,
    DROP_CORPSE = true,
    BASEANGBAND = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,3} },
  },
  body = bodies.quadruped,
  speed = 0,
  vision = 12,
  alertness = 30,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'GIANT_PINK_FROG',
  name = [[Giant pink frog]],
  display = 'R',
  color = colors.C_r,
  desc = [[It looks poisonous.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    random_move=50,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 8,
  ac = 16,
  max_life = resolvers.dice(5, 8),
  rarity = 1,
  level = 7,
  exp_worth = 16,
  flags = {
    ANIMAL = true,
    WILD_SHORE = true,
    MORTAL = true,
    WILD_SWAMP = true,
    WILD_ONLY = true,
    BASH_DOOR = true,
    DROP_CORPSE = true,
    BASEANGBAND = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.LOSE_STR, method=AttackType.BITE, dam={2,4} },
  },
  body = bodies.quadruped,
  speed = 0,
  vision = 12,
  alertness = 50,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'THE_WOUNDED_BEAR',
  name = [[The wounded bear]],
  display = 'q',
  color = colors.C_r,
  desc = [[A wounded bear, who has occasionally attacked humans.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 10,
  ac = 35,
  max_life = 110,
  rarity = 1,
  level = 12,
  exp_worth = 30,
  unique = true,
  flags = {
    ANIMAL = true,
    MORTAL = true,
    WILD_ONLY = true,
    FORCE_SLEEP = true,
    WILD_WOOD = true,
    WILD_MOUNTAIN = true,
    BASH_DOOR = true,
    UNIQUE = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    FORCE_MAXHP = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={1,10} },
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={1,10} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,11} },
  },
  body = bodies.quadruped,
  speed = 0,
  vision = 10,
  alertness = 10,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'BLOODFANG_THE_WOLF',
  name = [[Bloodfang, the Wolf]],
  display = 'C',
  color = colors.C_R,
  desc = [[It has been terrorising the nearby villages.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 6,
  ac = 30,
  max_life = 48,
  rarity = 1,
  level = 9,
  exp_worth = 30,
  unique = true,
  flags = {
    ANIMAL = true,
    MORTAL = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    BASH_DOOR = true,
    UNIQUE = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
    FORCE_MAXHP = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,12} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,12} },
  },
  body = bodies.quadruped,
  speed = 10,
  vision = 30,
  alertness = 20,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'EAGLE',
  name = [[Eagle]],
  display = 'B',
  color = colors.C_u,
  desc = [[A magnificent huge predatory bird.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 9,
  ac = 25,
  max_life = resolvers.dice(9, 9),
  rarity = 2,
  level = 12,
  exp_worth = 22,
  flags = {
    WILD_WASTE = true,
    ANIMAL = true,
    MORTAL = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    WILD_MOUNTAIN = true,
    CAN_FLY = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
    HAS_EGG = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={1,3} },
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={1,3} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={1,6} },
  },
  body = bodies.winged,
  speed = 10,
  vision = 30,
  alertness = 10,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'TANGLEWEED',
  name = [[Tangleweed]],
  display = '#',
  color = colors.C_g,
  desc = [[A mass of vegetation.  As you pass near it, it reaches out tendrils to ensnare you.  You can just make out skeletons of its previous victims deep within the thickets.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 5,
  ac = 5,
  max_life = resolvers.dice(5, 5),
  rarity = 4,
  level = 10,
  exp_worth = 10,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    DROP_60 = true,
    WILD_WOOD = true,
    SUSCEP_FIRE = true,
    STUPID = true,
    EMPTY_MIND = true,
    NO_CUT = true,
    KILL_TREES = true,
    NEVER_MOVE = true,
    NO_FEAR = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    ANIMAL = true,
    FORCE_SLEEP = true,
    FRIENDS = true,
    NO_SLEEP = true,
    COLD_BLOOD = true,
    NO_CONF = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PARALYZE, method=AttackType.HIT },
    { type=DamageType.PARALYZE, method=AttackType.HIT },
  },
  body = bodies.none,
  speed = -10,
  vision = 5,
  alertness = 5,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'POISON_IVY',
  name = [[Poison ivy]],
  display = '#',
  color = colors.C_g,
  desc = [[A mass of vegetation.  It seems to be growing...]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 5,
  ac = 5,
  max_life = resolvers.dice(5, 5),
  rarity = 4,
  level = 10,
  exp_worth = 10,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    DROP_60 = true,
    WILD_WOOD = true,
    SUSCEP_FIRE = true,
    STUPID = true,
    EMPTY_MIND = true,
    NO_CUT = true,
    KILL_TREES = true,
    NEVER_MOVE = true,
    NO_FEAR = true,
    WILD_GRASS = true,
    BASEANGBAND = true,
    ANIMAL = true,
    FORCE_SLEEP = true,
    FRIENDS = true,
    NO_SLEEP = true,
    COLD_BLOOD = true,
    NO_CONF = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.POISON, method=AttackType.HIT, dam={2,2} },
    { type=DamageType.POISON, method=AttackType.HIT, dam={2,2} },
  },
  spells = {
    MULTIPLY = true,
  },
  body = bodies.none,
  speed = -10,
  vision = 5,
  alertness = 5,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'GIANT_VENUS_FLYTRAP',
  name = [[Giant Venus Flytrap]],
  display = '#',
  color = colors.C_g,
  desc = [[A carnivorous plant that is difficult to detect, until it suddenly snaps shut around its prey and releases paralysing enzymes to stop its struggles.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 10,
  ac = 5,
  max_life = resolvers.dice(10, 10),
  rarity = 5,
  level = 15,
  exp_worth = 30,
  flags = {
    ANIMAL = true,
    WILD_SWAMP = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    CHAR_MULTI = true,
    SUSCEP_FIRE = true,
    STUPID = true,
    EMPTY_MIND = true,
    NO_CUT = true,
    NEVER_MOVE = true,
    CHAR_CLEAR = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PARALYZE, method=AttackType.HIT, dam={3,3} },
    { type=DamageType.PARALYZE, method=AttackType.HIT, dam={3,3} },
    { type=DamageType.PARALYZE, method=AttackType.HIT, dam={3,3} },
  },
  body = bodies.none,
  speed = 10,
  vision = 20,
  alertness = 0,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'HUORN',
  name = [[Huorn]],
  display = '#',
  color = colors.C_g,
  desc = [[A very strong near-sentient tree, which has become hostile to other living things.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    spell_prob = 9,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 10,
  ac = 45,
  max_life = resolvers.dice(50, 10),
  rarity = 1,
  level = 19,
  exp_worth = 75,
  flags = {
    WEIRD_MIND = true,
    ANIMAL = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    DROP_60 = true,
    IM_COLD = true,
    SUSCEP_FIRE = true,
    NO_SLEEP = true,
    NO_CUT = true,
    RES_WATE = true,
    NEVER_MOVE = true,
    BASEANGBAND = true,
    NO_CONF = true,
  },
  drop = { treasure=30, combat=30, magic=30, tool=5 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CRUSH, dam={3,6} },
    { type=DamageType.PHYSICAL, method=AttackType.CRUSH, dam={3,6} },
    { type=DamageType.PHYSICAL, method=AttackType.CRUSH, dam={3,6} },
    { type=DamageType.PHYSICAL, method=AttackType.CRUSH, dam={3,6} },
  },
  spells = {
    BLINK = true,
    TELE_TO = true,
  },
  body = bodies.none,
  speed = 0,
  vision = 40,
  alertness = 20,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'GREAT_EAGLE',
  name = [[Great eagle]],
  display = 'B',
  color = colors.C_u,
  desc = [[Greater and more intelligent than most of its kind, this great eagle is a messenger between the forces of good.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 5,
  ac = 65,
  max_life = resolvers.dice(100, 5),
  rarity = 2,
  level = 20,
  exp_worth = 150,
  flags = {
    ANIMAL = true,
    WILD_WASTE = true,
    WILD_ONLY = true,
    WILD_WOOD = true,
    WILD_MOUNTAIN = true,
    CAN_FLY = true,
    GOOD = true,
    BASEANGBAND = true,
    WILD_VOLCANO = true,
    DROP_CORPSE = true,
    HAS_EGG = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={6,3} },
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={6,3} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={3,6} },
  },
  body = bodies.winged,
  speed = 10,
  vision = 20,
  alertness = 20,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'MENELDOR_THE_SWIFT',
  name = [[Meneldor the Swift]],
  display = 'B',
  color = colors.C_u,
  desc = [[Among all the eagles of Middle-earth he is the swiftest, and in his time has borne messages between all of the Wise.  It was Meneldor who bore the Ring-bearer away from the destruction of Mount Doom.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 10,
  ac = 65,
  max_life = 800,
  rarity = 6,
  level = 24,
  exp_worth = 360,
  unique = true,
  flags = {
    ANIMAL = true,
    WILD_ONLY = true,
    WILD_MOUNTAIN = true,
    CAN_FLY = true,
    UNIQUE = true,
    PET = true,
    GOOD = true,
    BASEANGBAND = true,
    FORCE_MAXHP = true,
    DROP_CORPSE = true,
    WILD_VOLCANO = true,
    CAN_SPEAK = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={7,3} },
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={7,3} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={3,7} },
  },
  body = bodies.winged,
  speed = 30,
  vision = 20,
  alertness = 20,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'GWAIHIR_THE_WINDLORD',
  name = [[Gwaihir the Windlord]],
  display = 'B',
  color = colors.C_u,
  desc = [[The greatest of eagles in the Third Age of Middle-earth, Gwaihir rescued Gandalf the Wizard from Orthanc, and has twice brought his flock to the aid of Sauron's enemies in battle - first outside the gates of Erebor in the Battle of Five Armies, and then before the Black Gate of Mordor itself.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 10,
  ac = 65,
  max_life = 850,
  rarity = 6,
  level = 24,
  exp_worth = 360,
  unique = true,
  flags = {
    ANIMAL = true,
    WILD_ONLY = true,
    WILD_MOUNTAIN = true,
    CAN_FLY = true,
    UNIQUE = true,
    PET = true,
    GOOD = true,
    BASEANGBAND = true,
    FORCE_MAXHP = true,
    DROP_CORPSE = true,
    WILD_VOLCANO = true,
    CAN_SPEAK = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={15,2} },
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={15,2} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={3,10} },
  },
  body = bodies.winged,
  speed = 20,
  vision = 20,
  alertness = 20,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'THORONDOR_LORD_OF_EAGLES',
  name = [[Thorondor, Lord of Eagles]],
  display = 'B',
  color = colors.C_u,
  desc = [[Among the mightiest of birds, Thorondor is the messenger of the Valar, and brings news of Middle-earth to Valinor itself.  Nothing that can be seen from the airs of the world is hidden from him.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 12,
  ac = 65,
  max_life = 1020,
  rarity = 6,
  level = 30,
  exp_worth = 555,
  unique = true,
  flags = {
    ANIMAL = true,
    WILD_ONLY = true,
    WILD_MOUNTAIN = true,
    CAN_FLY = true,
    UNIQUE = true,
    PET = true,
    GOOD = true,
    BASEANGBAND = true,
    FORCE_MAXHP = true,
    DROP_CORPSE = true,
    WILD_VOLCANO = true,
    CAN_SPEAK = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={16,2} },
    { type=DamageType.PHYSICAL, method=AttackType.CLAW, dam={16,2} },
    { type=DamageType.PHYSICAL, method=AttackType.BITE, dam={4,10} },
  },
  body = bodies.winged,
  speed = 20,
  vision = 20,
  alertness = 20,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'HUMAN_WARRIOR',
  name = [[Human Warrior]],
  display = 'p',
  color = colors.C_u,
  desc = [[They are used as the main assault force of the human kings. They will help you in some quests.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 8,
  ac = 10,
  max_life = resolvers.dice(9, 8),
  rarity = 20,
  level = 0,
  exp_worth = 0,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    MALE = true,
    ONLY_GOLD = true,
    FRIENDS = true,
    BASH_DOOR = true,
    DROP_1D2 = true,
    PET = true,
    HAS_LITE = true,
    OPEN_DOOR = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.HIT, dam={5,8} },
  },
  body = bodies.humanoid,
  speed = 0,
  vision = 10,
  alertness = 0,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'ELVEN_ARCHER',
  name = [[Elven archer]],
  display = 'h',
  color = colors.C_W,
  desc = [[They are used as the main assault force of the elven kings. They will help you in some quests.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
    spell_prob = 1,
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 7,
  ac = 10,
  max_life = resolvers.dice(9, 7),
  rarity = 20,
  level = 0,
  exp_worth = 0,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    MALE = true,
    ONLY_GOLD = true,
    FRIENDS = true,
    BASH_DOOR = true,
    DROP_1D2 = true,
    PET = true,
    HAS_LITE = true,
    OPEN_DOOR = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.HIT, dam={2,8} },
  },
  spells = {
    ARROW_2 = true,
    ARROW_1 = true,
    ARROW_2 = true,
    ARROW_1 = true,
  },
  body = bodies.humanoid,
  speed = 0,
  vision = 10,
  alertness = 0,
  resolvers.mon_global_speed()
}

newEntity {
  define_as = 'DWARVEN_WARRIOR',
  name = [[Dwarven warrior]],
  display = 'h',
  color = colors.C_U,
  desc = [[They are used as the main assault force of the dwarven kings. They will help you in some quests.]],
  ai = 'tome2',
  ai_state = {
    sleep = resolvers.mon_sleep(),
  },
  stats = { STR=10, INT=10, WIS=10, DEX=10, CON=10, CHA=10 },
  hit_die = 8,
  ac = 10,
  max_life = resolvers.dice(8, 8),
  rarity = 20,
  level = 0,
  exp_worth = 0,
  flags = {
    MORTAL = true,
    WILD_ONLY = true,
    MALE = true,
    ONLY_GOLD = true,
    FRIENDS = true,
    BASH_DOOR = true,
    DROP_1D2 = true,
    PET = true,
    HAS_LITE = true,
    OPEN_DOOR = true,
    BASEANGBAND = true,
    DROP_CORPSE = true,
  },
  drop = { treasure=0, combat=0, magic=0, tool=0 },
  attacks = {
    { type=DamageType.PHYSICAL, method=AttackType.HIT, dam={6,8} },
  },
  body = bodies.humanoid,
  speed = 0,
  vision = 10,
  alertness = 0,
  resolvers.mon_global_speed()
}


-- seen dam types:  POISON PHYSICAL LOSE_STR PARALYZE
-- seen hit msgs:  CRUSH CLAW HIT BITE
-- seen body types:
--    none [0,0,0,0,0,0]:  4 (first seen on Tangleweed)
--    quadruped [0,1,0,2,1,0]:  5 (first seen on Wild rabbit)
--    winged [0,1,1,0,1,0]:  6 (first seen on Chaffinch)
--    humanoid [1,1,1,2,1,1]:  4 (first seen on Woodsman)
