-- $Id: bases.lua 81 2012-09-20 04:27:31Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Check that objects have same level of identification.
local function same_identified(o1, o2)
  if not o1.identified and not o2.identified then return true end
  if not o1.identified or not o2.identified then return false end
  return o1.identified == o2.identified
end

local function weapon_name_cb(self, name, t)
  name = name .. string.format(' (%dd%d)', self.combat[1], self.combat[2])
  if self.identified then
    name = name .. string.format(' (%+d,%+d)', self.combat[3], self.combat[4])
  end
  return name
end

local function launcher_name_cb(self, name, t)
  if self.identified then
    name = name .. string.format(' (%+d,%+d)', self.combat[3], self.combat[4])
  end
  name = name .. string.format(' (x%d)', self.ammo_mult)
  return name
end

local function armor_name_cb(self, name, t)
  if self.identified then
    name = name .. string.format(' [%d,%+d]', self.armor, self.armor_adj or 0)
  else
    name = name .. string.format(' [%d]', self.armor)
  end
  return name
end

newEntity {
  define_as = 'ESSENCE_BASE',
  type = 'misc',
  subtype = 'alchemic',
  base_tmpl = '& Essence~ of #',
  display = '*',
  color = colors.C_G,
  weight = 0.4,
  encumber = 0.4,
  cost = 2,
  stacking = true,
}

newEntity {
  define_as = 'CHEST_BASE',
  type = 'misc',
  subtype = 'chest',
  display = '~',
  color = colors.C_s,
}

newEntity {
  define_as = 'PARCHMENT_BASE',
  type = 'misc',
  subtype = 'parchment',
  display = '?',
  weight = 0.5,
  encumber = 0.5,
  stacking = true,
}

newEntity {
  define_as = 'CORPSE_BASE',
  type = 'misc',
  subtype = 'corpse',
  display = '~',
  color = colors.C_U,
  level = 20,
}

newEntity {
  define_as = 'INSTRUMENT_BASE',
  type = 'equipment',
  subtype = 'instrument',
  slot = 'LAUNCHER',
  display = '/',
  color = colors.C_W,
  weight = 3,
  encumber = 3,
  cost = 400,
}

newEntity {
  define_as = 'BOOMERANG_BASE',
  type = 'throwing',
  subtype = 'boomerang',
  slot = 'LAUNCHER',
  display = '{',
  name_cb = weapon_name_cb,
  color = colors.C_y,
  weight = 6,
  encumber = 6,
  stacking = true,
}

newEntity {
  define_as = 'AMMO_BASE',
  type = 'ammo',
  slot = 'QUIVER',
  display = '{',
  name_cb = weapon_name_cb,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Check identical everything else
    return true
  end
}

newEntity {
  base = 'AMMO_BASE',
  define_as = 'ARROW_BASE',
  subtype = 'arrows',
  weight = 0.2, -- override as necessary
  encumber = 0.2,
  stacking = true,
}

newEntity {
  base = 'AMMO_BASE',
  define_as = 'BOLT_BASE',
  subtype = 'bolts',
  weight = 0.3, -- override as necessary
  encumber = 0.3,
}

newEntity {
  base = 'AMMO_BASE',
  define_as = 'SHOT_BASE',
  subtype = 'shots',
  weight = 0.4,
  encumber = 0.4,
}

newEntity {
  define_as = 'LAUNCHER_BASE',
  type = 'shooter',
  slot = 'LAUNCHER',
  display = '}',
  stacking = true,
  name_cb = launcher_name_cb,
  check_stack = function(self, o)
    -- TODO Allow {average} pseudo-ID'd armor to stack.
    if not same_identified(self, o) then return false end
    -- TODO Check for identical enchantments.
    return config.settings.tome2.stack_wpn_armor
  end
}

newEntity {
  base = 'LAUNCHER_BASE',
  define_as = 'BOW_BASE',
  subtype = 'bow',
  color = colors.C_U,
  ammo_type = 'arrows',
}

newEntity {
  base = 'LAUNCHER_BASE',
  define_as = 'CROSSBOW_BASE',
  subtype = 'crossbow',
  color = colors.C_s,
  ammo_type = 'bolts',
}

newEntity {
  define_as = 'TOOL_BASE',
  type = 'tool',
  slot = 'TOOL',
}

newEntity {
  base = 'TOOL_BASE',
  define_as = 'DIGGER_BASE',
  subtype = 'digger',
  display = '\\',
  stacking = true,
}

newEntity {
  define_as = 'WEAPON_BASE',
  type = 'weapon',
  slot = 'MAIN_HAND',
  name_cb = weapon_name_cb,
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Check for identical enchantments.
    return config.settings.tome2.stack_wpn_armor
  end
}

newEntity {
  base = 'WEAPON_BASE',
  define_as = 'HAFTED_BASE',
  subtype = 'hafted',
  display = '\\',
}

newEntity {
  base = 'WEAPON_BASE',
  define_as = 'POLEARM_BASE',
  subtype = 'polearm',
  display = '/',
}

newEntity {
  base = 'WEAPON_BASE',
  define_as = 'SWORD_BASE',
  subtype = 'sword',
  display = '|',
}

newEntity {
  base = 'WEAPON_BASE',
  define_as = 'AXE_BASE',
  subtype = 'axe',
  display = '/',
  color = colors.C_s,
}

newEntity {
  define_as = 'ARMOR_BASE',
  type = 'armor',
  name_cb = armor_name_cb,
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Check for identical enchantments.
    return config.settings.tome2.stack_wpn_armor
  end
}

newEntity {
  base = 'ARMOR_BASE',
  define_as = 'BOOTS_BASE',
  type = 'armor',
  subtype = 'footwear',
  slot = 'BOOTS',
  display = ']',
}

newEntity {
  base = 'ARMOR_BASE',
  define_as = 'GLOVES_BASE',
  type = 'armor',
  subtype = 'handwear',
  slot = 'GLOVES',
  display = ']',
}

newEntity {
  base = 'ARMOR_BASE',
  define_as = 'HELM_BASE',
  type = 'armor',
  subtype = 'headwear',
  slot = 'HELM',
  display = ']',
}

newEntity {
  base = 'HELM_BASE',
  define_as = 'CROWN_BASE',
  armor = 0,
}

newEntity {
  base = 'ARMOR_BASE',
  define_as = 'SHIELD_BASE',
  type = 'armor',
  subtype = 'shield',
  slot = 'OFF_HAND',
  display = ')',
}

newEntity {
  base = 'ARMOR_BASE',
  define_as = 'CLOAK_BASE',
  type = 'armor',
  subtype = 'cloak',
  slot = 'CLOAK',
  display = '(',
}

newEntity {
  base = 'ARMOR_BASE',
  define_as = 'BODY_ARMOR_BASE',
  slot = 'BODY',
}

newEntity {
  base = 'BODY_ARMOR_BASE',
  define_as = 'SOFT_BODY_ARMOR_BASE',
  subtype = 'light',
  display = '(',
}

newEntity {
  base = 'BODY_ARMOR_BASE',
  define_as = 'HARD_BODY_ARMOR_BASE',
  subtype = 'heavy',
  display = '[',
}

newEntity {
  base = 'HARD_BODY_ARMOR_BASE',
  define_as = 'DRAGON_ARMOR_BASE',
  is_dragon_armor = true,
}

newEntity {
  define_as = 'LITE_BASE',
  type = 'equipment',
  subtype = 'light',
  slot = 'LITE',
  display = '~',
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Require same turns of light left.
    return true
  end
}

newEntity {
  define_as = 'AMULET_BASE',
  type = 'jewelry',
  subtype = 'amulet',
  base_tmpl = '& Amulet~ of #',
  slot = 'AMULET',
  display = '"',
  weight = 0.2,
  encumber = 0.2,
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Check same everything else
    return true
  end
}

newEntity {
  define_as = 'RING_BASE',
  type = 'jewelry',
  subtype = 'ring',
  base_tmpl = '& Ring~ of #',
  slot = 'RING',
  display = '=',
  weight = 0.2,
  encumber = 0.2,
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Check same everything else
    return true
  end
}

newEntity {
  define_as = 'TRAP_BASE',
  type = 'misc',
  subtype = 'trapkit',
  base_tmpl = '& # Trap Set',
  display = '`',
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Check for identical enchantments.
    -- Interestingly, this uses the 'Allow weapons and armor to stack'
    -- option.
    return config.settings.tome2.stack_wpn_armor
  end
}

newEntity {
  define_as = 'STAFF_BASE',
  type = 'utility',
  subtype = 'staff',
  display = '_',
  color = colors.C_d,
  weight = 5,
  encumber = 5,
  stacking = true,
  check_stack = function(self, o)
    -- TODO require same_identified() or both known empty
    -- TODO require same number of charges
    -- TODO require same spell and same [M|N] numbers
    return false
  end
}

newEntity {
  define_as = 'WAND_BASE',
  type = 'utility',
  subtype = 'wand',
  display = '-',
  color = colors.C_d,
  weight = 1,
  encumber = 1,
  stacking = true,
  check_stack = function(self, o)
    -- TODO require same_identified() or both known empty
    -- TODO require same spell and same [M|N] numbers
    return false
  end
}

newEntity {
  define_as = 'ROD_TIP_BASE',
  type = 'utility',
  subtype = 'rodtip',
  base_tmpl = '& Rod Tip~ of #',
  display = '-',
  color = colors.C_d,
  weight = 1.5,
  encumber = 1.5,
  stacking = true,
}

newEntity {
  define_as = 'ROD_BASE',
  type = 'utility',
  subtype = 'rod',
  display = '-',
}

newEntity {
  define_as = 'SCROLL_BASE',
  type = 'utility',
  subtype = 'scroll',
  base_tmpl = '& Scroll~ of #',
  display = '?',
  color = colors.C_d,
  weight = 0.5,
  encumber = 0.5,
  stacking = true,
}

newEntity {
  define_as = 'POTION_BASE',
  type = 'utility',
  subtype = 'potion',
  base_tmpl = '& Potion~ of #',
  display = '!',
  weight = 0.4,
  encumber = 0.4,
  on_quaff = function(actor)
    game.log('[not implemented]')
  end,
  stacking = true,
}

newEntity {
  define_as = 'FOOD_BASE',
  type = 'food',
  subtype = 'misc',
  display = ',',
  stacking = true,
}

newEntity {
  base = 'FOOD_BASE',
  define_as = 'MUSHROOM_BASE',
  subtype = 'mushroom',
  base_tmpl = '& Mushroom~ of #',
}

newEntity {
  define_as = 'MONEY_BASE',
  type = 'money',
  subtype = 'money',
  money_value = 0,
  name_cb = function(self, name, t)
    -- TODO This could be done better...
    return self.money_value .. ' gold pieces'
  end,
  display = '$',
  weight = 0,
  encumber = 0,
  auto_pickup = true,
  on_prepickup = function(self, who, id)
    if who == game.player then
      who:adjGold(self.money_value)
      game.logPlayer(who, 'You pick up %d gold pieces.', self.money_value)
      game.level.map:removeObject(who.x, who.y, id)
      return true
    else
      return false
    end
  end,
}

newEntity {
  define_as = 'RUNE_BASE',
  type = 'rune',
  base_tmpl = '& Rune~ [#]',
  display = '?',
  weight = 0.2,
  encumber = 0.2,
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- Everything stacks but runestones.
    return self.define_as ~= 'RUNE_RUNESTONE' and o.define_as ~= 'RUNE_RUNESTONE'
  end
}

newEntity {
  base = 'RUNE_BASE',
  define_as = 'RUNE_DAM_BASE',
  subtype = 'damage',
}

newEntity {
  base = 'RUNE_BASE',
  define_as = 'RUNE_MOD_BASE',
  subtype = 'modifier',
}

newEntity {
  define_as = 'BOOK_BASE',
  type = 'magic',
  subtype = 'spellbook',
  display = '?',
  weight = 3,
  encumber = 3,
  stacking = true,
  check_stack = function(self, o)
    if not same_identified(self, o) then return false end
    -- TODO Require same spells.  Also don't let the Mage quest book stack
    -- with anything else.
    return false
  end
}
