-- $Id: options.lua 80 2012-09-20 04:24:58Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

newOption {
  key = 'linear_stats',
  name = 'Linear stats display',
  group = 'display',
  show_in_birther = true,
  desc = [[Controls how stat values over 18 are displayed.  If enabled, stats will be formatted linearly from 18.1 through 40; if disabled, they will be shown in "percentile" format from 18/01 through 18/220 (which is shown as 18/***).]],
  default = false,
}

newOption {
  key = 'exp_display',
  name = 'Experience display',
  group = 'display',
  default = 'Relative',
  desc = [[Controls how experience points are displayed.  '#00FFFF#Absolute#LAST#' simply displays your current experience points.  '#00FFFF#Relative#LAST#' displays the number of points needed for the next level.  '#00FFFF#Percent#LAST#' displays experience as a fraction of the difference between this level and the next level.]],
  values = { 'Absolute', 'Relative', 'Percent' },
  short_values = { ['Absolute']='Abs', ['Relative']='Rel', ['Percent']='Pct' }
}

newOption {
  key = 'stack_wpn_armor',
  name = 'Allow weapons and armor to stack',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, armor and weapons of the same type with identical enchantments and the same identified state will be combined into into a single inventory slot.  Attempting to wield a stack of weapons or wear a stack of armor pieces will unstack one of them and wear/wield that.]],
}

newOption {
  key = 'maximize',
  name = 'Maximize mode',
  group = 'birth',
  desc = [[Controls how race-, subrace- and class-based initial stat modifications are applied at character creation time.  Normally, a character's stats cannot be raised above 18/100 (28 in linear display) without magical equipment.  If this option is disabled, this remains true; birth modifications are only applied to initial stat values, and the stat maxima are left at 18/100.
  
If this option is enabled, on the other hand, birth modifications are applied to both initial and maximum stat values.  Thus, for instance, a Human Warrior, with +5 to STR and -2 to INT, will have a maximum STR of 18/150 and a maximum INT of 18/80 (33 resp. 26 in linear display).  You may thus want to disable this option if your choice of race, subrace and class has many net negative stat modifiers.]],
  default = true,
}

newOption {
  key = 'preserve',
  name = 'Preserve artifacts',
  group = 'birth',
  desc = [[{insert description here}]],
  default = true,
  placeholder = true,
}

newOption {
  key = 'fates',
  name = 'Allow fates',
  group = 'birth',
  desc = [[If enabled, you will receive random fates, ranging from good (a fate to find a particular powerful artifact on a certain level) to mediocre (a fate to encounter some mundane monster or object on a certain level) to bad (a fate to die on a certain level).]],
  default = true,
}

newOption {
  key = 'n_quests',
  name = 'Random quests',
  group = 'birth',
  desc = [[The number of random quests that will appear in the "mainline" dungeons (Barrow Downs, Mirkwood, Mordor, Angband).  Can range from 0 (if you want no random quests) to 98 (one on every level of the mainline dungeons).]],
  default = 20,
  min = 0,
  max = 98,
  placeholder = true,
}
