-- $Id: combat.lua 2 2012-07-08 02:40:16Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

newSkill {
  name = 'Combat',
  short_name = 'COMBAT',
  desc = [[General ability to fight and to pseudo-id armour and weapons.  It also allows the use of heavier armour without penalties]],
  rate = 1000,
  depth = 0,
  player_calc = function(p, init)
    if init then
      p.to_h, p.to_h_melee, p.to_d, p.to_d_melee = 0, 0, 0, 0
      p.skill_fighting = 0
      return
    end
    local sk_c = p:getSkillLevel('COMBAT')
    local sk_m = p:getSkillLevel(p:getSkillForMeleeStyle())
    p.skill_fighting = math.floor(5*(7*sk_m + 3*sk_c)/10)
    p.to_d = p:getSkillLevel('COMBAT', 10)
  end,
}

newSkill {
  name = 'Weaponmastery',
  short_name = 'WEAPONMASTERY',
  desc = [[General ability to use melee weapons]],
  rate = 1000,
  depth = 1,
  parent = 'COMBAT',
  ripple_to = {
    COMBAT = 50,
  },
}

local function wpn_skill(p, skill, sub)
  local wpns = p:getInven('MAIN_HAND')
  if #wpns == 0 then return end
  for _, wpn in ipairs(wpns) do
    if wpn.subtype ~= sub then return end
  end
  -- All wielded weapons are of the appropriate type; apply bonuses.
  local sk = p:getSkillLevel(skill)
  p.to_h_melee = p.to_h_melee + sk
  p.to_d_melee = p.to_d_melee + math.floor(sk/2)
end

newSkill {
  name = 'Sword-mastery',
  short_name = 'SWORD_MASTERY',
  desc = [[Ability to use swords]],
  rate = 1000,
  depth = 2,
  parent = 'WEAPONMASTERY',
  ripple_to = {
    WEAPONMASTERY = 25,
    COMBAT = 7,
  },
  player_calc = function(p, init)
    if not init then wpn_skill(p, 'SWORD_MASTERY', 'sword') end
  end
}

newSkill {
  name = 'Critical-hits',
  short_name = 'CRITICAL_HITS',
  desc = [[Ability to deal critical hits with swords < 5lb]],
  rate = 1000,
  depth = 3,
  parent = 'SWORD_MASTERY',
  ripple_to = {
    SWORD_MASTERY = 5,
  },
}

newSkill {
  name = 'Axe-mastery',
  short_name = 'AXE_MASTERY',
  desc = [[Ability to use axes]],
  rate = 1000,
  depth = 2,
  parent = 'WEAPONMASTERY',
  ripple_to = {
    WEAPONMASTERY = 25,
    COMBAT = 7,
  },
  player_calc = function(p, init)
    if not init then wpn_skill(p, 'AXE_MASTERY', 'axe') end
  end
}

newSkill {
  name = 'Hafted-mastery',
  short_name = 'HAFTED_MASTERY',
  desc = [[Ability to use hafted weapons]],
  rate = 1000,
  depth = 2,
  parent = 'WEAPONMASTERY',
  ripple_to = {
    WEAPONMASTERY = 25,
    COMBAT = 7,
  },
  player_calc = function(p, init)
    if not init then wpn_skill(p, 'HAFTED_MASTERY', 'hafted') end
  end
}

newSkill {
  name = 'Stunning-blows',
  short_name = 'STUNNING_BLOWS',
  desc = [[Ability to stun opponents when doing critical hits with hafted weapons > 5 lb]],
  rate = 1000,
  depth = 3,
  parent = 'HAFTED_MASTERY',
  ripple_to = {
    HAFTED_MASTERY = 5,
  },
}

newSkill {
  name = 'Polearm-mastery',
  short_name = 'POLEARM_MASTERY',
  desc = [[Ability to use polearms]],
  rate = 1000,
  depth = 2,
  parent = 'WEAPONMASTERY',
  ripple_to = {
    WEAPONMASTERY = 25,
    COMBAT = 7,
  },
  player_calc = function(p, init)
    if not init then wpn_skill(p, 'POLEARM_MASTERY', 'polearm') end
  end
}

newSkill {
  name = 'Archery',
  short_name = 'ARCHERY',
  desc = [[General ability to use ranged weapons]],
  rate = 1000,
  depth = 1,
  parent = 'COMBAT',
  ripple_to = {
    COMBAT = 50,
  },
}

newSkill {
  name = 'Sling-mastery',
  short_name = 'SLING_MASTERY',
  desc = [[Ability to use slings]],
  rate = 1000,
  depth = 2,
  parent = 'ARCHERY',
  mkey = 23,
  action = [[Fire piercing shots]],
  ripple_to = {
    ARCHERY = 25,
    COMBAT = 7,
  },
}

newSkill {
  name = 'Bow-mastery',
  short_name = 'BOW_MASTERY',
  desc = [[Ability to use bows]],
  rate = 1000,
  depth = 2,
  parent = 'ARCHERY',
  mkey = 23,
  action = [[Fire piercing shots]],
  ripple_to = {
    ARCHERY = 25,
    COMBAT = 7,
  },
}

newSkill {
  name = 'Crossbow-mastery',
  short_name = 'CROSSBOW_MASTERY',
  desc = [[Ability to use crossbows]],
  rate = 1000,
  depth = 2,
  parent = 'ARCHERY',
  mkey = 23,
  action = [[Fire piercing shots]],
  ripple_to = {
    ARCHERY = 25,
    COMBAT = 7,
  },
}

newSkill {
  name = 'Boomerang-mastery',
  short_name = 'BOOMERANG_MASTERY',
  desc = [[Ability to use boomerangs]],
  rate = 1000,
  depth = 2,
  parent = 'ARCHERY',
  ripple_to = {
    ARCHERY = 25,
    COMBAT = 7,
  },
}

newSkill {
  name = 'Barehand-combat',
  short_name = 'BAREHAND_COMBAT',
  desc = [[Ability to fight barehanded]],
  rate = 1000,
  depth = 1,
  parent = 'COMBAT',
  ripple_to = {
    COMBAT = 50,
  },
  random_gain = 70,
}

newSkill {
  name = 'Bearform-combat',
  short_name = 'BEARFORM_COMBAT',
  desc = [[Ability to fight in bear form]],
  rate = 1000,
  depth = 1,
  parent = 'COMBAT',
  visible = function() return false end,
}

newSkill {
  name = 'Boulder-throwing',
  short_name = 'BOULDER_THROWING',
  desc = [[Ability to make and throw boulders]],
  rate = 1000,
  depth = 1,
  parent = 'COMBAT',
  mkey = 21,
  action = [[Tear down a wall to create boulders]],
  ripple_to = {
    COMBAT = 40,
  },
}

newSkill {
  name = 'Antimagic',
  short_name = 'ANTIMAGIC',
  desc = [[Ability to generate an antimagic field]],
  rate = 1000,
  depth = 1,
  parent = 'COMBAT',
  mkey = 3,
  action = [[Use antimagic]],
  random_gain = 80,
  exclude = { MAGIC_DEVICE, MANA, GEOMANCY, FIRE, AIR, WATER, EARTH, CONVEYANCE, DIVINATION, TEMPORAL, META, MIND, NATURE, UDUN, SORCERY, DEMONOLOGY, RUNECRAFT, NECROMANCY, MINDCRAFT, MUSIC, PRAYER, DRUIDISTIC, THAUMATURGY },
}
