-- $Id: timed_effects.lua 2 2012-07-08 02:40:16Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

local Stats = require "engine.interface.ActorStats"

local function msg(actor, text)
  text = text:gsub('#Target', actor.name:capitalize())
	     :gsub('#target', actor.name)
  game.logSeen(actor, text)
end

local function fly(actor, msg)
  if game.flyers and actor.x and actor.y and game.level.map.seens(actor.actor.y) then
    local sx, sy = game.level.map:getTileToScreen(actor.x, actor.y)
    if game.level.map.seens(actor.x, actor.y) then
      game.flyers:add(sx, sy, 20, (rng.range(0, 2) - 1) * 0.5, msg, {255,100,80})
    end
  end
end

--[[
newEffect{
	name = "ACIDBURN",
	desc = "Burning from acid",
	type = "physical",
	status = "detrimental",
	parameters = { power=1 },
	on_gain = function(self, err) return "#Target# is covered in acid!", "+Acid" end,
	on_lose = function(self, err) return "#Target# is free from the acid.", "-Acid" end,
	on_timeout = function(self, eff)
		DamageType:get(DamageType.ACID).projector(eff.src or self, self.x, self.y, DamageType.ACID, eff.power)
	end,
}
]]

newEffect {
  name = 'BLIND',
  desc = 'Blinded',
  short_desc = 'Blind',
  type = 'physical',
  status = 'detrimental',
  on_gain = function(self, p)
    return '#Target# is blinded!', '+Blind'
  end,
  on_lose = function(self, p)
    return '#Target# can see again', '-Blind'
  end,
}

newEffect {
  name = 'POISON',
  desc = 'Poisoned',
  short_desc = 'Poison',
  type = 'physical',
  status = 'detrimental',
  on_gain = function(self, p)
    return '#Target# is poisoned!', '+Pois'
  end,
  on_lose = function(self, p)
    return '#Target# is no longer poisoned.', '-Pois'
  end,
  on_timeout = function(self, p)
    DamageType:get(DamageType.POISON).projector(p.src or self, self.x, self.y, DamageType.POISON, 1)
  end
}

local stun_msgs = {
  [0] = { '#Target# is no longer stunned.',	nil,		'-Stun' },
  [1] = { '#Target# is stunned.',		'+Stun',	'-HvyStun' },
  [2] = { '#Target# is heavily stunned.',	'+HvyStun',	'-KnockOut' },
  [3] = { '#Target# is knocked out.',		'+KnockOut',	nil },
}

newEffect {
  name = 'STUN',
  desc = 'Stunned',
  short_desc = 'Stun',
  type = 'physical',
  status = 'detrimental',
  on_gain = function(self, p)
    p.lvl = math.min(math.floor((cur.dur + 49)/50), 3)
    p.short_desc = stun_msgs[lvl][1]:sub(2)
    return stun_msg[p.lvl][1], stun_msg[p.lvl][2]
  end,
  on_lose = function(self, p)
    return stun_msg[0][1], stun_msg[0][3]
  end,
  on_merge = function(self, cur, p)
    local lvl = math.min(math.floor((cur.dur + p.dur + 49)/50), 3)
    cur.dur = cur.dur + p.dur
    if lvl > cur.lvl then
      cur.lvl = lvl
      msg(self, stun_msgs[lvl][1])
      fly(self, stun_msgs[lvl][2])
    end
    return cur
  end,
  on_timeout = function(self, p)
    local lvl = math.min(math.floor((p.dur + 48)/50), 3)
    if lvl < p.lvl then
      p.lvl = lvl
      fly(self, stun_msgs[lvl][3])
    end
  end,
}
