-- $Id: grids.lua 48 2012-08-28 01:19:20Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

load('/data/general/grids/all.lua')

newEntity {
  base = 'UP',
  define_as = 'EXIT_BREE_HOUSE_QUEST',
  change_zone = 'town-bree',
  change_level = 1,
}

-- TODO Make this available for other aggravation effects.
local function aggravate_near(src)
  local any_woken, any_hasted = false, false
  for _, e in pairs(game.level.entities) do
    -- Skip non-monsters, dead monsters, and the source.
    if e.ai_state and e.ai_state.sleep and not e.dead and e ~= src then
      -- Wake up anything within 2*MAX_SIGHT.
      if core.fov.distance(e.x, e.y, src.x, src.y) < 40 then
	e.ai_state.sleep = 0
	e.ai_target.actor = game:getPlayer()
	e.changed = true
	any_woken = true
      end
      -- Speed up monsters in LoS.
      if game.level.map.seens(e.x, e.y) and e.speed and e.base_speed then
	e.speed = math.max(e.speed, e.base_speed + 10)
	e:computeGlobalSpeed()
	-- TODO Potentially anger pets.
	e.changed = true
	any_hasted = true
      end
    end
  end
  if any_hasted then
    game.log('You feel a sudden stirring nearby!')
  elseif any_woken then
    game.log('You hear a sudden stirring in the distance!')
  end
end

newEntity {
  base = 'LOCKED_DOOR_POWER_6',
  define_as = 'ALARM_DOOR',
  on_door_open = function(self, e)
    game.log("#FF0000#An alarm rings!#LAST#");
    aggravate_near(e)
    while true do
      local spot = game.level:pickSpotRemove{ type='quest', subtype='rigged-door' }
      if not spot then break end
      game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.OPEN_DOOR)
    end
    game.log("#FF0000#The door explodes!#LAST#");
    game.level.map(self.x, self.y, engine.Map.TERRAIN, game.zone.grid_list.FLOOR)
  end
}
