-- $Id: zone.lua 48 2012-08-28 01:19:20Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

return {
  name = 'Bree',
  level_scheme = 'fixed',
  max_level = 1,
  width = 198,
  height = 66,
  persistent = 'zone',
  all_remembered = true,
  all_lited = true,
  day_night = true,

  generator = {
    map = {
      class = 'engine.generator.map.Static',
      map = 'towns/bree'
    },
    actor = {
      class = 'engine.generator.actor.Random',
      nb_npc = { 10, 10 }
    },
    object = {
      class = 'engine.generator.object.Random',
      nb_object = { 0, 0 },
    },
  },

  entry_point = function(_, _, from_zone)
    if not from_zone then
      return nil
    elseif from_zone.name == 'Barrow Downs' then 
      return game.level:pickSpot{ type='zone-change', subtype='barrow-downs' }
    elseif from_zone.name == "Thieves' Lair" then
      return game.level:pickSpot{ type='quest', subtype='house' }
    else
      -- TODO Return spots for the Trolls quest and Wights quest, for
      -- entering from the world map, and possibly for recalling back to
      -- town from the dungeon.
      return nil
    end
  end,

  on_enter = function(_, _, _)
    -- Close off the Trolls or Wights quest if we've finished them.
    local q_troll = game.player:hasQuest('bree-trolls')
    if q_troll and q_troll:isCompleted() and q_troll.close_entrance then
      local spot = game.level:pickSpot{ type='quest', subtype='trolls' }
      game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.GRASS)
      q_troll.close_entrance = nil
    end

    local q_wight = game.player:hasQuest('bree-wights')
    if q_wight and q_wight:isCompleted() and q_wight.close_entrance then
      local spot = game.level:pickSpot{ type='quest', subtype='wights' }
      game.level.map(spot.x, spot.y, engine.Map.TERRAIN, game.zone.grid_list.DIRT)
      q_wight.close_entrance = nil
    end
  end
}
