-- $Id: Home.lua 46 2012-08-28 01:09:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'
local Store = require 'engine.Store'
local Dialog = require 'engine.Dialog'
local ShowStore = require 'engine.dialogs.ShowStore'

module(..., package.seeall, class.inherit(Store))

_M.homes_def = {}

function _M:loadHomes(f)
  self.homes_def = self:loadList(f)
end

function _M:init(t, no_default)
  Store.init(self, t, no_default)
  -- There's no obvious way to override the 't.body = {INVEN=10000}' in
  -- Store.init(), so we patch that up here.
  self:getInven('INVEN').max = 24
end

function _M:canRestock()
  -- Homes don't restock.
  return false
end

function _M:loadup(level, zone, force_nb)
  -- Nor should they be filled with an initial stock.
end

-- Mostly the same as Store.interact(), with a few tweaks.
function _M:interact(who, name)
  local store, inven = self:getInven("INVEN"), who:getInven("INVEN")
  local d
  local action_cb = function(what, o, item)
    if what == "buy" then
      if o:getNumber() > 1 then
	local q = GetQuantity.new(nil, nil, o:getNumber(), o:getNumber(), function(qty) self:doBuy(who, o, item, qty, d) end)
	game:registerDialog(q)
      else
	self:doBuy(who, o, item, 1, d)
      end
    else
      if o:getNumber() > 1 then
	local q
	q = GetQuantity.new(nil, nil, o:getNumber(), o:getNumber(), function(qty) self:doSell(who, o, item, qty, d) end)
	game:registerDialog(q)
      else
	self:doSell(who, o, item, 1, d)
      end
    end
  end
  local desc_cb = function(what, o) return self:descObject(who, what, o) end
  local descprice_cb = function(what, o) return '' end
  d = ShowStore.new((name or self.name), store, inven, nil, nil,
		    action_cb, desc_cb, descprice_cb,
		    self.allow_sell, self.allow_buy,
		    function(item) end, self, who)
  game:registerDialog(d)
end

function _M:doBuy(who, o, item, qty, store_dialog)
  qty = math.min(qty, o:getNumber())
  self:transfer(self, who, item, qty)
  if store_dialog then store_dialog:updateStore() end
  game.log('%s gets from home:  %s', who.name, o:getName{ force_qty=qty })
end

function _M:doSell(who, o, item, qty, store_dialog)
  qty = math.min(qty, o:getNumber())
  local o_name = o:getName{ force_qty=qty }
  self:transfer(who, self, item, qty)
  if store_dialog then store_dialog:updateStore() end
  game.log('%s puts in home:  %s', who.name, o:getName{ force_qty=qty })
end
