-- $Id: NPC.lua 2 2012-07-08 02:40:16Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require "engine.class"
local ActorAI = require "engine.interface.ActorAI"
local Faction = require "engine.Faction"
local AttackType = require 'mod.class.AttackType'
require "mod.class.Actor"

module(..., package.seeall, class.inherit(mod.class.Actor, engine.interface.ActorAI))

function _M:init(t, no_default)
  mod.class.Actor.init(self, t, no_default)
  ActorAI.init(self, t)

  self.max_level = 150
end

function _M:act()
  while self:enoughEnergy() and not self.dead do
    -- Do basic actor stuff
    if not mod.class.Actor.act(self) then return end
    local old_energy = self.energy.value

    -- Compute FOV, if needed
    self:computeFOV(self.sight or 20)

    -- Let the AI think .... beware of Shub !
    -- If AI did nothing, use energy anyway
    self:doAI()
    if not self.energy.used then self:useEnergy() end
    if old_energy == self.energy.value then break end -- Prevent infinite loops
  end
end

--- Called by ActorLife interface
-- We use it to pass aggression values to the AIs
function _M:onTakeHit(value, src)
  if not self.ai_target.actor and src.targetable then
    self.ai_target.actor = src
  end

  return mod.class.Actor.onTakeHit(self, value, src)
end

function _M:tooltip()
  local str = mod.class.Actor.tooltip(self)
  return str..([[

  Target: %s
  UID: %d]]):format(
  self.ai_target.actor and self.ai_target.actor.name or "none",
  self.uid)
end

function _M:attackTarget(target)
  for _, atk in ipairs(self.attacks or {}) do
    -- If we died from retaliatory damage, stop attacking.
    if self.dead then break end
    AttackType:monAttackTarget(self, target, atk)
  end
  self:useEnergy()
end

function _M:getExpChart(level)
  return level*level*level * 6
end

function _M:levelup()
  local newlev = self:levelupMsg()

  if rng.range(1, 10) <= 8 then
    -- Gain hp
    self.max_life = self.max_life + self.hit_die
    self.life = self.life + self.hit_die
  end
  if rng.range(1, 10) <= 4 then
    -- Gain speed
    self.speed = self.speed + rng.range(1, 2)
  end
  if rng.range(1, 10) <= 5 then
    -- Increase AC
    self.ac = self.ac + math.max(math.floor(self.ac/10), 1)
  end
  if rng.range(1, 10) <= 3 then
    -- Increase attack power
    for _ = 1, 20 do
      local atk = rng.table(self.attacks or {})
      if atk.dam then
	atk.dam[1] = atk.dam[1] + 1
	break
      end
    end
  end
end
