-- $Id: Object.lua 34 2012-08-26 22:53:40Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require "engine.class"
require "engine.Object"
require "engine.interface.ObjectActivable"
require "engine.interface.ObjectIdentify"

module(..., package.seeall, class.inherit(
	engine.Object,
	engine.interface.ObjectActivable,
	engine.interface.ObjectIdentify))

function _M:init(t, no_default)
	t.encumber = t.encumber or 0

	engine.Object.init(self, t, no_default)
	engine.interface.ObjectActivable.init(self, t)
	engine.interface.ObjectIdentify.init(self, t)
end

function _M:getName(t)
  t = t or {}
  local qty = t.force_qty or self:getNumber()
  local name = self.display_name or self.name
  if self.name_cb then name = self:name_cb(name, t) end
  if self.base_tmpl then name = self.base_tmpl:gsub('#', name) end
  if qty > 1 then
    name = name:gsub('~', 's'):gsub('^&', qty)
  else
    name = name:gsub('~', '')
    if self.unique then
      name = name:gsub('^& ', 'The ')
    else
      name = name:gsub('^& ([AEIOUaeiou])', 'an %1'):gsub('^& ', 'a ')
    end
  end
  if t.capitalize then name = string.capitalize(name) end
  return name
end

function _M:getPrice(stack)
  -- TODO Increase price for object properties
  return (stack and self:getNumber() or 1) * self.cost
end

function _M:setNumber(n)
  local qty = self:getNumber()
  if qty < n then
    local o = self:clone()
    -- Don't need to get recursive here...
    o.stacked = nil
    for _ = qty + 1, n do
      self:stack(o:clone())
    end
  elseif qty > n then
    for _ = n + 1, qty do
      self:unstack()
    end
  end
end

function _M:canStack(o)
  if not self.stacking or not o.stacking then return false end
  -- The 'define_as' field should be unique across object tval/svals.
  if self.define_as ~= o.define_as then return false end
  -- Slight Hack(TM) for quest objects.
  if self.quest_obj and o.quest_obj then
    if self.quest_obj ~= o.quest_obj then return false end
  elseif self.quest_obj or o.quest_obj then
    return false
  end
  -- TODO check for matching ego/artifact stuff
  -- TODO check for matching cursed status
  if self.check_stack then return self:check_stack(o) end
  -- That should cover all objections.
  return true
end

function _M:isCursed()
  -- TODO check for curses applied from outside, or heavy/perma-curse.
  return self.flags and self.flags.CURSED
end
