-- $Id: Combat.lua 12 2012-07-10 23:44:57Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require "engine.class"
local DamageType = require "engine.DamageType"
local Map = require "engine.Map"
local Target = require "engine.Target"

--- Interface to add ToME combat system
module(..., package.seeall, class.make)

--- Checks what to do with the target
-- Talk ? attack ? displace ?
function _M:bumpInto(target)
  local reaction = self:reactionToward(target)
  if reaction < 0 then
    return self:attackTarget(target)
  elseif reaction >= 0 then
    if self.move_others then
      -- Displace
      game.level.map:remove(self.x, self.y, Map.ACTOR)
      game.level.map:remove(target.x, target.y, Map.ACTOR)
      game.level.map(self.x, self.y, Map.ACTOR, target)
      game.level.map(target.x, target.y, Map.ACTOR, self)
      self.x, self.y, target.x, target.y = target.x, target.y, self.x, self.y
    end
  end
end

--- Makes the death happen!
-- [This is done in NPC and Player now.]
--[[
function _M:attackTarget(target)
  if self.combat then
    local dam = self.combat.dam + self:getStat('STR') - target.combat_armor
    DamageType:get(DamageType.PHYSICAL).projector(self, target.x, target.y, DamageType.PHYSICAL, math.max(0, dam))
  end

  -- We use up our own energy
  self:useEnergy(game.energy_to_act)
end
]]

function _M:checkHit(target, power)
  -- Hit or miss, attacking an NPC wakes it up.
  if target.ai_state and target.ai_state.sleep then
    target.ai_state.sleep = 0
  end

  print('[COMBAT] checkHit('..power..')')
  -- Fixed 5% miss chance and 5% hit chance.
  local edge = rng.range(1, 20)
  if edge <= 2 then
    print('[COMBAT] checkHit:  edge case 5% '..(edge == 1 and 'hit' or 'miss'))
    return edge == 1
  end

  if self == game.player then
    print('[COMBAT] checkHit:  attacker is player')
  elseif target == game.player then
    print('[COMBAT] checkHit:  target is player')
  end
  local tgt_ac = target:getArmor()
  local atk_pow = power
  if self == game.player then
    atk_pow = atk_pow + self:getLuckScale(-10, 10)
  else
    atk_pow = atk_pow + 3*math.min(self.level, 1)
    if target == game.player then
      atk_pow = atk_pow - target:getLuckScale(-10, 10)
    end
  end
  print('[COMBAT] checkHit:  power '..atk_pow..' vs. ac '..tgt_ac)
  -- TODO Penalize for invisible target
  return atk_pow > 0 and rng.range(1, atk_pow) > (3*tgt_ac)/4
end
