-- $Id: PlayerSkills.lua 2 2012-07-08 02:40:16Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'

module(..., package.seeall, class.make)

_M.skill_step = 1000
_M.skill_max = 50*_M.skill_step
_M.skills_def = {}
_M.skills_toplevel = {}
_M.abilities_def = {}
_M.abilities_order = {}

function _M:loadDefinition(file, env)
  env = env or {
    Skills = self,
    newSkill = function(t) self:newSkill(t) end,
    newAbility = function(t) self:newAbility(t) end,
    load = function(f) self:loadDefinition(f) end
  }
  local f, err = util.loadfilemods(file, setmetatable(env, {__index = _G}))
  if not f and err
    then error(err)
    os.exit()
  end
  local ok, err = pcall(f)
  if not ok and err then error(err) end
end

function _M:newSkill(t)
  assert(t.name, 'no skill name')
  t.short_name = t.short_name or t.name:upper():gsub('[^a-zA-Z0-9_]', '_')
  self.skills_def[t.short_name] = t

  if t.parent then
    assert(self.skills_def[t.parent], 'undefined parent '..t.parent)
    local parent = self.skills_def[t.parent]
    parent.children = parent.children or {}
    table.insert(parent.children, t.short_name)
  else
    table.insert(self.skills_toplevel, t.short_name)
  end
end

function _M:newAbility(t)
  assert(t.name, 'no ability name')
  assert(t.cost, 'no ability cost')
  t.short_name = t.short_name or t.name:upper():gsub('[^a-zA-Z0-9_]', '_')
  self.abilities_def[t.short_name] = t
  table.insert(self.abilities_order, t.short_name)
end

function _M:init(t)
  self.skill_points = 0
  self.skills = {}
  self.abilities = {}
  self.bought_abilities = {}
  for tag, def in pairs(self.skills_def) do
    local t = {
      value = 0,
      mod = 0,
      name = def.name,
      desc = def.desc,
    }
    self.skills[tag] = t
  end
end

function _M:getSkillLevel(tag, scale)
  local sk = self.skills[tag]
  if not sk then return 0 end
  if not scale then return math.floor(sk.value / _M.skill_step) end
  return math.floor((sk.value * scale) / _M.skill_max)
end
