-- $Id: load.lua 82 2012-09-20 04:30:24Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- This file loads the game module, and loads data
local KeyBind = require "engine.KeyBind"
local DamageType = require "engine.DamageType"
local AttackType = require 'mod.class.AttackType'
local ActorResource = require "engine.interface.ActorResource"
--local ActorTalents = require "engine.interface.ActorTalents"
local ActorInventory = require 'engine.interface.ActorInventory'
local PlayerSkills = require 'mod.class.interface.PlayerSkills'
local ActorAI = require "engine.interface.ActorAI"
local ActorTemporaryEffects = require "engine.interface.ActorTemporaryEffects"
local Birther = require "engine.Birther"
local GameOptions = require 'mod.dialogs.GameOptions'
local Store = require 'mod.class.Store'
local StoreOwner = require 'mod.class.StoreOwner'
local Home = require 'mod.class.Home'
local MathomHouse = require 'mod.class.MathomHouse'

-- Initialize settings.
GameOptions:loadDefinition('/data/options.lua')
GameOptions:setupOptions()

-- The 16 predefined colors from the original *_info.txt files, for easy
-- translation.
defineColor('C_d', 0x00, 0x00, 0x00)
defineColor('C_w', 0xFF, 0xFF, 0xFF)
defineColor('C_s', 0x80, 0x80, 0x80)
defineColor('C_o', 0xFF, 0x80, 0x00)
defineColor('C_r', 0xC0, 0x00, 0x00)
defineColor('C_g', 0x00, 0x80, 0x00)
defineColor('C_b', 0x00, 0x00, 0xFF)
defineColor('C_u', 0x80, 0x40, 0x00)
defineColor('C_D', 0x40, 0x40, 0x40)
defineColor('C_W', 0xC0, 0xC0, 0xC0)
defineColor('C_v', 0xFF, 0x00, 0xFF)
defineColor('C_y', 0xFF, 0xFF, 0x00)
defineColor('C_R', 0xFF, 0x00, 0x00)
defineColor('C_G', 0x00, 0xFF, 0x00)
defineColor('C_B', 0x00, 0xFF, 0xFF)
defineColor('C_U', 0xC0, 0x80, 0x40)

-- Useful keybinds
KeyBind:load("move,hotkeys,inventory,actions,interface,tome2,debug")

-- Damage types
DamageType:loadDefinition("/data/damage_types.lua")
AttackType:loadDefinition('/data/attack_types.lua')

-- Talents
--ActorTalents:loadDefinition('/data/talents.lua')

-- Skills and abilities
PlayerSkills:loadDefinition('/data/skills/skills.lua')

-- Timed Effects
ActorTemporaryEffects:loadDefinition("/data/timed_effects.lua")

dofile('/mod/resolvers.lua')

-- Actor resources
ActorResource:defineResource('Mana', 'mana', nil, 'mana_regen', 'Mental power, used to cast spells or perform some innate abilities.')
ActorResource:defineResource('Piety', 'piety', nil, 'piety_regen', 'Spiritual power, used to cast spells granted to you by your god.', -300000, 300000)

-- Actor inventory
ActorInventory:defineInventory('MAIN_HAND', 'Main hand', true, 'Weapon')
ActorInventory:defineInventory('OFF_HAND', 'Off hand', true, 'Shield')
ActorInventory:defineInventory('BODY', 'Armor', true, 'Body armor')
ActorInventory:defineInventory('CLOAK', 'Cloak', true, 'Cloak')
ActorInventory:defineInventory('LAUNCHER', 'Launcher', true, 'Bow, crossbow or sling')
ActorInventory:defineInventory('QUIVER', 'Quiver', true, 'Arrows, crossbow bolts or sling shots')
ActorInventory:defineInventory('GLOVES', 'Hands', true, 'Gloves or gauntlets')
ActorInventory:defineInventory('BOOTS', 'Feet', true, 'Boots')
ActorInventory:defineInventory('HELM', 'Head', true, 'Helmets or other headwear')
ActorInventory:defineInventory('RING', 'Fingers', true, 'Rings')
ActorInventory:defineInventory('AMULET', 'Neck', true, 'Amulet')
ActorInventory:defineInventory('LITE', 'Light', true, 'Torches or lantern')
ActorInventory:defineInventory('TOOL', 'Tool', true, 'Shovel or other tool')
ActorInventory:defineInventory('SYMBIOTE', 'Symbiote', true, 'A hypnotized non-moving creature worn on your body.  Requires Symbiosis skill.')

-- Actor AIs
ActorAI:loadDefinition("/engine/ai/")
ActorAI:loadDefinition('/mod/class/ai/')

-- Birther descriptor
Birther:loadDefinition("/data/birth/descriptors.lua")

-- Stores
StoreOwner:loadOwners('/data/general/stores/owners.lua')
Store:loadStores('/data/general/stores/stores.lua')
Home:loadHomes('/data/general/stores/homes.lua')
MathomHouse:loadMuseums('/data/general/stores/mathom-houses.lua')

-- Questionable Hack(TM):  We'd like this to live in Game, but it gets
-- called during the require'ing of modules that get require'd directly or
-- indirectly while Game is still in the middle of being require'd, so that
-- doesn't work.
_G.registerOnTurn = function(f)
  _G.turn_hooks = _G.turn_hooks or {}
  table.insert(_G.turn_hooks, f)
end
return { require "mod.class.Game" }
