-- $Id: resolvers.lua 82 2012-09-20 04:30:24Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

function resolvers.store(def, type)
  return { __resolver='store', def, type or 'store' }
end
function resolvers.calc.store(t, e)
  e.block_move = function(self, x, y, who, act, couldpass)
    if who and who.player and act then
      local name = self.name
      if self.store.owner then
	name = ('%s [%s (%d)]'):format(name, self.store.owner.name, self.store.owner.max_cost)
      end
      self.store:loadup(game.level, game.zone)
      self.store:interact(who, name)
    end
    return true
  end
  e.store = game:getStore(t[1], t[2])
  -- Delete the origin field
  return nil
end

function resolvers.store_owner(candidates)
  return { __resolver='store_owner', candidates }
end
function resolvers.calc.store_owner(t, e)
  return game:getStoreOwner(rng.table(t[1]))
end

function resolvers.building_chat(name)
  return { __resolver='building_chat', name }
end
function resolvers.calc.building_chat(t, e)
  e.chat = t[1]
  e.block_move = function(self, x, y, who, act, couldpass)
    if who and who.player and act then
      local chat = (require 'engine.Chat').new(self.chat, self, who, {npc=self, player=who})
      chat:invoke()
    end
    return true
  end
  -- Delete the origin field
  return nil
end

function resolvers.birth_equip(t)
  return { __resolver='birth_equip', __resolve_last=true, t }
end
function resolvers.calc.birth_equip(t, e)
  for _, filter in ipairs(t[1]) do
    local o = game.zone:makeEntity(game.level, 'object', filter, nil, true)
    if o then
      -- TODO Make this full ID when that's implemented.
      o:identify(true)
      if filter.qty then
	local n
	if type(filter.qty) == 'table' then
	  n = rng.dice(filter.qty[1], filter.qty[2])
	elseif type(filter.qty) == 'number' then
	  n = filter.qty
	end
	o:setNumber(n)
      end
      local ro = e:wearObject(o, true, false)
      if ro then
	if type(ro) == 'table' then e:addObject(e.INVEN_INVEN, ro) end
      elseif not ro then
	e:addObject(e.INVEN_INVEN, o)
      end
    end
  end
  e:sortInven()
  -- Delete the origin field
  return nil
end

function resolvers.birth_mana_mult(mult)
  return { __resolver='birth_mana_mult', mult }
end
function resolvers.calc.birth_mana_mult(t, e)
  e.mana_mult = e.mana_mult * t[1];
  -- Delete the origin field
  return nil
end

function resolvers.birth_stat_apply()
  return { __resolver='birth_stat_apply', __resolve_last=true }
end
function resolvers.calc.birth_stat_apply(t, e)
  for id, delta in pairs(e.adj_stat or {}) do
    if config.settings.tome2.maximize then
      local v = e:getStat(id, { raw='birth_adj' })
      e:setStat(id, { birth_adj=v+delta })
    else
      local v = e:getStat(id, { raw='base' })
      e:setStat(id, { base=v+delta*10, cur=v+delta*10 })
    end
  end
  -- Clean this up behind us.
  e.adj_stat = nil
  -- Delete the origin field
  return nil
end

function resolvers.birth_skill_adj(t)
  return { __resolver='birth_skill_adj', t }
end
function resolvers.calc.birth_skill_adj(t, e)
  for key, adj in pairs(t[1]) do
    local sk = e.skills[key]
    if not sk then
      print('[RESOLVER] ??? bad skill '..key)
    else
      sk.value = sk.value + (adj.base_adj or 0)
      sk.mod = sk.mod + (adj.mod_adj or 0)
      if adj.base_set then sk.value = adj.base_set end
      if adj.mod_set then sk.mod = adj.mod_set end
    end
  end
  -- Delete the origin field
  return nil
end

function resolvers.birth_gain_at_lev(t)
  return { __resolver='birth_gain_at_lev', t }
end
function resolvers.calc.birth_gain_at_lev(t, e)
  e.gain_at_level = e.gain_at_level or {}
  for lev, spec in ipairs(t[1]) do
    e.gain_at_level[lev] = e.gain_at_level[lev] or {}
    table.merge(e.gain_at_level[lev], spec, true)
  end
  -- Delete the origin field
  return nil
end

function resolvers.birth_hp_precalc()
  return { __resolver='birth_hp_precalc', __resolve_last=true }
end
function resolvers.calc.birth_hp_precalc(t, e)
  -- Constrain the final total base HP at level 50.
  local min_l50_hp = math.floor((e.max_level * (e.hit_die - 1) * 3)/8)
  local max_l50_hp = math.floor((e.max_level * (e.hit_die - 1) * 5)/8)

  local hp = { [1] = e.hit_die }
  while true do
    for lev = 2, 50 do
      hp[lev] = hp[lev-1] + rng.range(1, e.hit_die)
    end
    local hp_l50 = hp[e.max_level]
    if hp_l50 >= min_l50_hp and hp_l50 <= max_l50_hp then break end
  end

  e.base_hp_by_lev = hp
  -- Delete the origin field
  return nil
end

function resolvers.mon_sleep()
  return { __resolver='mon_sleep' }
end
function resolvers.calc.mon_sleep(t, e)
  if e.alertness > 0 then
    return e.alertness*2 + rng.range(1, e.alertness*10)
  else
    return 0
  end
end

function resolvers.mon_global_speed()
  return { __resolver='mon_global_speed' }
end
function resolvers.calc.mon_global_speed(t, e)
  e:computeGlobalSpeed()
  -- Keep current and initial speed around for later hasting.
  e.base_speed = e.speed
  return e.speed
end
